import pytest
import os
import tempfile

import wfuzz


def get_temp_file():
    temp_name = next(tempfile._get_candidate_names())
    defult_tmp_dir = tempfile._get_default_tempdir()

    return os.path.join(defult_tmp_dir, temp_name)


def test_filter_prev_payload():

    filename = get_temp_file()
    for res in wfuzz.get_session(
        "-z range --zD 0-0 -H test:1 -u http://localhost:9000/anything/FUZZ"
    ).fuzz(save=filename):
        pass

    filename_new = get_temp_file()
    for res in wfuzz.get_session(
        "-z wfuzzp --zD {} -u FUZZ -H test:2 --oF {}".format(filename, filename_new)
    ).fuzz(save=filename_new):
        pass

    assert (
        len(
            list(
                wfuzz.get_session(
                    "-z wfuzzp --zD {} --slice r.headers.request.test=2 --dry-run -u FUZZ".format(
                        filename_new
                    )
                ).fuzz()
            )
        )
        == 1
    )
    assert (
        len(
            list(
                wfuzz.get_session(
                    "-z wfuzzp --zD {} --slice FUZZ[r.headers.request.test]=1 --dry-run -u FUZZ".format(
                        filename_new
                    )
                ).fuzz()
            )
        )
        == 1
    )



from wfuzz.factories.plugin_factory import plugin_factory
from wfuzz.fuzzobjects import FuzzPlugin

from queue import Queue


def test_sum_plugin_output(example_full_fuzzres):
    plugin = plugin_factory.create("plugin_from_summary", "a message")

    assert plugin.is_visible(True) is False
    assert plugin.is_visible(False) is True


def test_find_plugin_output_from_factory():
    plugin = plugin_factory.create(
        "plugin_from_finding",
        "a plugin",
        "a source",
        "an issue",
        "some data",
        FuzzPlugin.INFO,
    )

    assert plugin.is_visible(True) is True
    assert plugin.is_visible(False) is False


def test_find_plugin_output(get_plugin):
    plugin = get_plugin("links")[0]
    plugin.results_queue = Queue()
    plugin.add_result("a source", "an issue", "some data", FuzzPlugin.INFO)

    plugin_res = plugin.results_queue.get()

    assert plugin_res.is_visible(True) is True
    assert plugin_res.is_visible(False) is False



import pytest


from wfuzz.fuzzrequest import FuzzRequest
from wfuzz.fuzzobjects import FuzzResult
from wfuzz.fuzzobjects import FPayloadManager
from wfuzz.filters.ppfilter import FuzzResFilter
from wfuzz.facade import Facade


@pytest.fixture
def full_fuzzres(request):
    raw_req, raw_resp = request.param
    fr = FuzzRequest()
    fr.update_from_raw_http(raw_req, "http", raw_resp, None)

    return FuzzResult(history=fr)


@pytest.fixture
def full_fuzzreq(request):
    raw_req, raw_resp = request.param
    fr = FuzzRequest()
    fr.update_from_raw_http(raw_req, "http", raw_resp, None)

    return fr


@pytest.fixture
def fuzzres_from_url(request):
    fr = FuzzRequest()
    fr.url = request.param

    return FuzzResult(history=fr)


@pytest.fixture
def filter_obj():
    return FuzzResFilter()


@pytest.fixture
def example_full_fuzzres():
    raw_req, raw_resp = (
        "GET /path?param1=1&param2=2 HTTP/1.1\n"
        "Host: www.wfuzz.org\n"
        "User-Agent: curl/7.58.0\n"
        "Accept: */*\n"
        "Cookie: cookie1=1\n",
        "HTTP/1.1 302 Found\n"
        "Content-Type: text/html; charset=utf-8\n"
        "Content-Language: en\n"
        "Location: https://wfuzz.readthedocs.io/en/latest/\n"
        "Vary: Accept-Language, Cookie\n"
        "Server: nginx/1.14.0 (Ubuntu)\n"
        "X-Fallback: True\n"
        "X-Served: Django\n"
        "X-Deity: web01\n"
        "Date: Wed, 23 Jan 2019 21:43:59 GMT\n"
        "Content-Length: 0\n"
        "Set-Cookie: name=Nicholas; expires=Sat, 02 May 2009 23:38:25 GMT\n",
    )
    fr = FuzzRequest()
    fr.update_from_raw_http(
        raw_req, "http", raw_resp, b"Some line\n and words\nasdsdas"
    )

    return FuzzResult(history=fr)


@pytest.fixture
def example_full_fuzzres_content(request):
    raw_content = request.param

    raw_req, raw_resp = (
        "GET /path?param1=1&param2=2 HTTP/1.1\n"
        "Host: www.wfuzz.org\n"
        "User-Agent: curl/7.58.0\n"
        "Accept: */*\n"
        "Cookie: cookie1=1\n",
        "HTTP/1.1 200 OK\n"
        "Content-Type: text/html; charset=utf-8\n"
        "Content-Language: en\n"
        "Vary: Accept-Language, Cookie\n"
        "Server: nginx/1.14.0 (Ubuntu)\n"
        "X-Fallback: True\n"
        "X-Served: Django\n"
        "X-Deity: web01\n"
        "Date: Wed, 23 Jan 2019 21:43:59 GMT\n"
        "Content-Length: 0\n"
        "Set-Cookie: name=Nicholas; expires=Sat, 02 May 2009 23:38:25 GMT\n",
    )
    fr = FuzzRequest()
    fr.update_from_raw_http(raw_req, "http", raw_resp, raw_content)

    fuzzres = FuzzResult(history=fr)
    fuzzres.payload_man = FPayloadManager()

    return fuzzres


@pytest.fixture
def example_full_fuzzres_no_response():
    raw_req = "GET /path?param1=1&param2=2 HTTP/1.1\nHost: www.wfuzz.org\nUser-Agent: curl/7.58.0\nAccept: */*\n"

    fr = FuzzRequest()
    fr.update_from_raw_http(raw_req, "http", None, None)

    return FuzzResult(history=fr)


@pytest.fixture
def get_plugin():
    def _get_customer_plugin(name):
        return [x() for x in Facade().scripts.get_plugins(name)]

    return _get_customer_plugin



import pytest


@pytest.mark.parametrize(
    "fuzzres_from_url, filter_string, expected_result",
    [
        (
            "http://www.wfuzz.org/path/test.php?param=1&param2=2",
            "r.urlp.scheme='http'",
            True,
        ),
        (
            "http://www.wfuzz.org/path/test.php?param=1&param2=2",
            "r.urlp.netloc='www.wfuzz.org'",
            True,
        ),
        (
            "http://www.wfuzz.org/path/test.php?param=1&param2=2",
            "r.urlp.path='/path/test.php'",
            True,
        ),
        (
            "http://www.wfuzz.org/path/test.php?param=1&param2=2",
            "r.urlp.ffname='test.php'",
            True,
        ),
        (
            "http://www.wfuzz.org/path/test.php?param=1&param2=2",
            "r.urlp.fname='test'",
            True,
        ),
        (
            "http://www.wfuzz.org/path/test.php?param=1&param2=2",
            "r.urlp.hasquery",
            True,
        ),
        (
            "http://www.wfuzz.org/path/test.php?param=1&param2=2",
            "not r.urlp.isbllist",
            True,
        ),
    ],
    indirect=["fuzzres_from_url"],
)
def test_urlp(filter_obj, fuzzres_from_url, filter_string, expected_result):
    assert filter_obj.is_visible(fuzzres_from_url, filter_string) == expected_result


@pytest.mark.parametrize(
    "fuzzres_from_url, filter_string, expected_result",
    [("http://www.wfuzz.org/path?param=1&param2=2", "r.is_path", False)],
    indirect=["fuzzres_from_url"],
)
def test_ispath(filter_obj, fuzzres_from_url, filter_string, expected_result):
    assert filter_obj.is_visible(fuzzres_from_url, filter_string) == expected_result


@pytest.mark.parametrize(
    "fuzzres_from_url, filter_string, expected_result",
    [
        (
            "http://www.wfuzz.org/path?param=1&param2=2",
            "r.pstrip",
            "http://www.wfuzz.org/path-gparam-gparam2",
        ),
    ],
    indirect=["fuzzres_from_url"],
)
def test_pstrip(filter_obj, fuzzres_from_url, filter_string, expected_result):
    assert filter_obj.is_visible(fuzzres_from_url, filter_string) == expected_result



#!/usr/bin/python
# -*- coding: utf-8 -*-

import copy
import os
import unittest
import tempfile

import wfuzz

LOCAL_DOMAIN = "http://localhost"
URL_LOCAL = "%s:8000/dir" % (LOCAL_DOMAIN)
HTTPD_PORT = 8000

ECHO_URL = "%s:8000/echo" % (LOCAL_DOMAIN)
HTTPBIN_URL = "http://localhost:9000"

REPLACE_HOSTNAMES = [
    ("localhost:8000", "httpserver:8000"),
    ("localhost:9000", "httpbin:80"),
    ("9000", "80"),
    ("localhost", "httpserver"),
]

# $ export PYTHONPATH=./src
# $ python -m unittest discover
#
# docker containers with HTTP server and proxy must be started before running these tests
# $ cd tests/server_dir
# $ docke-compose up

# IDEAS:
#
# baseline duplicated with single filters
# chain iterator duplicated with everything
# bad params
# duplicate with post instead of get
# conn delays?
# script args

testing_savedsession_tests = []

testing_tests = []

savedsession_tests = [
    # parse post params
    (
        "test_novalue_post_fuzz",
        "-z list --zD a -u {}/anything -d FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp --zD $$PREVFILE$$ -u FUZZ --filter r.params.post.a:=1 --field r.params.post.a",
        ["1"],
        None,
    ),
    (
        "test_json_post_fuzz2",
        '-z list --zD anything -u {}/FUZZ -d {{"a":"2"}} -H Content-Type:application/json'.format(
            HTTPBIN_URL
        ),
        "-z wfuzzp --zD $$PREVFILE$$ -u FUZZ --field r.params.post.a",
        ["2"],
        None,
    ),
    (
        "test_json_post_fuzz3",
        '-z list --zD anything -u {}/FUZZ -d {{"a":"2"}} -H Content-Type:application/json'.format(
            HTTPBIN_URL
        ),
        "-z wfuzzp --zD $$PREVFILE$$ -u FUZZ --filter r.params.post.a:=1 --field r.params.post.a",
        ["1"],
        None,
    ),
    (
        "test_json_nested",
        '-z list --zD anything -u {}/FUZZ -d {{"test":"me","another":1,"nested":{{"this":2}}}} -H Content-Type:application/json'.format(
            HTTPBIN_URL
        ),
        "-z wfuzzp --zD $$PREVFILE$$ -u FUZZ --field r.params.post.nested.this",
        ["2"],
        None,
    ),
    (
        "test_json_nested2",
        '-z list --zD anything -u {}/FUZZ -d {{"test":"me","another":1,"nested":{{"this":2}}}} -H Content-Type:application/json'.format(
            HTTPBIN_URL
        ),
        "-z wfuzzp --zD $$PREVFILE$$ -u FUZZ --field r.params.post.another",
        ["1"],
        None,
    ),
    # field fuzz values
    (
        "test_desc_fuzz",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ FUZZ",
        ["http://localhost:9000/1"],
        None,
    ),
    (
        "test_desc_attr",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ FUZZ[url]",
        ["http://localhost:9000/1"],
        None,
    ),
    (
        "test_desc_concat_number",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ FUZZ[url]FUZZ[c]",
        ["http://localhost:9000/1 - 404"],
        None,
    ),
    (
        "test_desc_url_number",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ http://localhost:FUZZ[c]/",
        ["http://localhost:9000/1 - 404"],
        "Pycurl error 7:",
    ),
    # set values
    (
        "test_desc_concat_number_slice",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ --slice r.c:=302 FUZZ[url]FUZZ[c]",
        ["http://localhost:9000/1 - 302"],
        None,
    ),
    (
        "test_desc_rewrite_url",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ --prefilter=r.url:=r.url|replace('1','2') FUZZ",
        ["http://localhost:9000/2"],
        None,
    ),
    (
        "test_desc_rewrite_url2",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ --slice r.url:=r.url|replace('1','2') FUZZ[url]",
        ["http://localhost:9000/2"],
        None,
    ),
    # fuzz value slice filters
    (
        "test_desc_concat_fuzz_symbol_op",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ --prefilter FUZZ[r.url]=+'2' FUZZ",
        ["http://localhost:9000/12"],
        None,
    ),
    (
        "test_fuzz_symbol_code",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ --slice FUZZ[c]=404 FUZZ",
        ["http://localhost:9000/1"],
        None,
    ),
    (
        "test_fuzz_value_code",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ --slice c=404 FUZZ",
        ["http://localhost:9000/1"],
        None,
    ),
    # fuzz value exceptions
    (
        "test_fuzz_symbol_code",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ --slice FUZ1Z[c]=404 FUZZ",
        ["http://localhost:9000/1"],
        "Unknown field",
    ),
    (
        "test_fuzz_symbol_code2",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ --slice FUZ2Z[c]=404 FUZZ",
        ["http://localhost:9000/1"],
        "Non existent FUZZ payload",
    ),
    (
        "test_desc_assign_fuzz_symbol_op",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ --slice r.url:=r.url|replace('1','2') FUZZ[url]",
        ["http://localhost:9000/2"],
        None,
    ),
    (
        "test_fuzz_param_int",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ --slice r.params.get:=2 FUZZ",
        ["http://localhost:9000/2"],
        "Non existent FUZZ payload",
    ),
    # filter based on various payloads
    (
        "test_fuzz_fuz2z_code",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ -z list,404-302-200 --prefilter FUZZ[code]=FUZ2Z FUZZ[url]/FUZ2Z",
        ["http://localhost:9000/1 - 404"],
        None,
    ),
    (
        "test_fuzz_fuz2z_code2",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ -z list,404-302-200 --prefilter FUZZ[code]=FUZ2Z FUZZ[url]",
        ["http://localhost:9000/1"],
        None,
    ),
    (
        "test_fuzz_fuz2z_code3",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ -z list,404-302-200 --prefilter FUZZ[code]=FUZ2Z FUZZ",
        ["http://localhost:9000/1"],
        None,
    ),
    # set values various payloads
    (
        "test_set_fuzz_from_fuz2z_full",
        "-z range,1-1 {}/FUZZ?param=1".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ -z list,6-3 --prefilter r.params.get.param:=FUZ2Z FUZZ",
        ["http://localhost:9000/1?param=6", "http://localhost:9000/1?param=3"],
        None,
    ),
    (
        "test_set_fuzz_from_fuz2z_full2",
        "-z range,1-1 {}/FUZZ?param=1".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ -z list,6-3 --prefilter FUZZ[r.params.get.param]:=FUZ2Z FUZZ",
        ["http://localhost:9000/1?param=6", "http://localhost:9000/1?param=3"],
        None,
    ),
    (
        "test_set_fuzz_from_fuz2z_full_all",
        "-z range,1-1 {}/FUZZ?param=1&param2=2".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ -z range,6-6 --prefilter r.params.all:=FUZ2Z FUZZ",
        ["http://localhost:9000/1?param=6&param2=6"],
        None,
    ),
    (
        "test_app_fuzz_from_fuz2z_full_all",
        "-z range,1-1 {}/FUZZ?param=1&param2=2".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ -z range,6-6 --prefilter r.params.all=+FUZ2Z FUZZ",
        ["http://localhost:9000/1?param=16&param2=26"],
        None,
    ),
    # fails ("test_set_fuzz_from_fuz2z_url", "-z range,1-1 {}/FUZZ?param=1".format(HTTPBIN_URL), "-z wfuzzp,$$PREVFILE$$ -z list,6-3 --prefilter r.params.get.param:=FUZ2Z FUZZ[url]", ["http://localhost:9000/1?param=6", "http://localhost:9000/1?param=3"], None),
    # test different field
    (
        "test_field",
        "-z range,1-1 {}/FUZZ".format(HTTPBIN_URL),
        "-z wfuzzp,$$PREVFILE$$ --field c FUZZ",
        ["404"],
        None,
    ),
]

basic_tests = [
    # different connect host ip
    # travis has an old pycurl version ("test_static_strquery_set_ip", "http://wfuzz.org/FUZZ?var=1&var2=2", [["anything"], ['PUT', 'GET', 'DELETE']], dict(connect_to_ip={'ip': '127.0.0.1', 'port': '9000'}, method='FUZ2Z', filter="content~'url' and content~'http://wfuzz.org'"), [(200, '/anything')] * 3, None),
    # encoding tests
    (
        "test_encode_cookie2_utf8_return",
        "%s/anything" % HTTPBIN_URL,
        [["は国"]],
        dict(
            cookie=["test=FUZZ"],
            filter="content~'test=\\\\u00e3\\\\u0081\\\\u00af\\\\u00e5\\\\u009b\\\\u00bd'",
        ),
        [(200, "/anything")],
        None,
    ),
    (
        "test_encode_header_utf8_return",
        "%s/headers" % HTTPBIN_URL,
        [["は国"]],
        dict(
            headers=[("myheader", "FUZZ")],
            filter="content~'Myheader' and content~'\\\\u00e3\\\\u0081\\\\u00af\\\\u00e5\\\\u009b\\\\u00bd'",
        ),
        [(200, "/headers")],
        None,
    ),
    (
        "test_encode_path",
        "%s/FUZZ" % HTTPBIN_URL,
        [["は国"]],
        dict(),
        [(404, "/は国")],
        None,
    ),
    (
        "test_encode_basic_auth",
        "%s/basic-auth/FUZZ/FUZZ" % HTTPBIN_URL,
        [["は国"]],
        dict(auth={"method": "basic", "credentials": "FUZZ:FUZZ"}),
        [(200, "/basic-auth/は国/は国")],
        None,
    ),
    # ("test_encode_postdata", "%s/anything" % HTTPBIN_URL, [["は国"]], dict(postdata="a=FUZZ", filter="content~'は国'"), [(200, '/anything')], None),
    (
        "test_encode_postdata",
        "%s/anything" % HTTPBIN_URL,
        [["は国"]],
        dict(postdata="a=FUZZ", filter="content~'\\\\u306f\\\\u56fd'"),
        [(200, "/anything")],
        None,
    ),
    (
        "test_encode_url_filter",
        "%s/FUZZ" % HTTPBIN_URL,
        [["は国"]],
        dict(filter="url~'は国'"),
        [(404, "/は国")],
        None,
    ),
    # ("test_encode_var", "%s/anything?var=FUZZ" % HTTPBIN_URL, [["は国"]], dict(filter="content~'\"は国\"'"), [(200, '/anything')], None),
    (
        "test_encode_var",
        "%s/anything?var=FUZZ" % HTTPBIN_URL,
        [["は国"]],
        dict(filter="content~'\"\\\\u306f\\\\u56fd\"'"),
        [(200, "/anything")],
        None,
    ),
    (
        "test_encode_redirect",
        "%s/redirect-to?url=FUZZ" % HTTPBIN_URL,
        [["は国"]],
        dict(
            filter="r.headers.response.Location='%C3%A3%C2%81%C2%AF%C3%A5%C2%9B%C2%BD'"
        ),
        [(302, "/redirect-to")],
        None,
    ),
    # ("test_encode_cookie", "%s/cookies" % HTTPBIN_URL, [["は国"]], dict(cookie=["cookie1=FUZZ"], follow=True, filter="content~FUZZ"), [(200, '/cookies')], None),
    (
        "test_encode_cookie",
        "%s/cookies" % HTTPBIN_URL,
        [["は国"]],
        dict(
            cookie=["cookie1=FUZZ"], follow=True, filter="content~'\\\\u306f\\\\u56fd'"
        ),
        [(200, "/cookies")],
        None,
    ),
    # postdata tests
    # pycurl does not allow it ("test_get_postdata", "%s/FUZZ?var=1&var2=2" % HTTPBIN_URL, [["anything"]], dict(postdata='a=1', filter="content~'\"form\":{\"a\":\"1\"}'"), [(200, '/anything')], None),
    (
        "test_allmethods_postdata",
        "%s/FUZZ?var=1&var2=2" % HTTPBIN_URL,
        [["anything"], ["PUT", "POST", "DELETE"], ["333888"]],
        dict(
            method="FUZ2Z",
            postdata="a=FUZ3Z",
            filter='content~FUZ2Z and content~\'"a": "\' and content~FUZ3Z',
        ),
        [(200, "/anything")] * 3,
        None,
    ),
    # httpbin extra tests
    (
        "test_gzip",
        "%s/FUZZ" % HTTPBIN_URL,
        [["gzip"]],
        dict(filter="content~'\"gzipped\": true'"),
        [(200, "/gzip")],
        None,
    ),
    (
        "test_response_utf8",
        "%s/encoding/FUZZ" % HTTPBIN_URL,
        [["utf8"]],
        dict(),
        [(200, "/encoding/utf8")],
        None,
    ),
    (
        "test_image",
        "%s/image/FUZZ" % HTTPBIN_URL,
        [["jpeg"]],
        dict(filter="content~'JFIF'"),
        [(200, "/image/jpeg")],
        None,
    ),
    (
        "test_deflate",
        "%s/FUZZ" % HTTPBIN_URL,
        [["deflate"]],
        dict(filter="content~'\"deflated\": true'"),
        [(200, "/deflate")],
        None,
    ),
    (
        "test_robots_disallow",
        "%s/FUZZ" % HTTPBIN_URL,
        [["robots.txt"]],
        dict(script="robots"),
        [(200, "/deny"), (200, "/robots.txt")],
        None,
    ),
    (
        "test_response_base64",
        "%s/base64/FUZZ" % HTTPBIN_URL,
        None,
        dict(
            filter="content~'HTTPBIN is awesome'",
            payloads=[("list", dict(values="HTTPBIN is awesome", encoder=["base64"]))],
        ),
        [(200, "/base64/SFRUUEJJTiBpcyBhd2Vzb21l")],
        None,
    ),
    # this does not work as you get the encoded value ("test_response_base64_FUZZ", "%s/base64/FUZZ" % HTTPBIN_URL, None, dict(filter="content~FUZZ", payloads=[("list", dict(values="HTTPBIN is awesome", encoder=["base64"]))]), [(200, '/base64/SFRUUEJJTiBpcyBhd2Vzb21l')], None),
    (
        "test_basic_auth",
        "%s/basic-auth/FUZZ/FUZZ" % HTTPBIN_URL,
        [["userpass"]],
        dict(auth={"method": "basic", "credentials": "FUZZ:FUZZ"}),
        [(200, "/basic-auth/userpass/userpass")],
        None,
    ),
    (
        "test_digest_auth",
        "%s/digest-auth/auth/FUZZ/FUZZ" % HTTPBIN_URL,
        [["userpass"]],
        dict(auth={"method": "digest", "credentials": "FUZZ:FUZZ"}),
        [(200, "/digest-auth/auth/userpass/userpass")],
        None,
    ),
    (
        "test_delayed_response",
        "%s/delay/FUZZ" % HTTPBIN_URL,
        [["2"]],
        dict(req_delay=1),
        [(200, "/delay/2")],
        "Operation timed out",
    ),
    (
        "test_static_strquery_set_multiple_method",
        "%s/FUZZ?var=1&var2=2" % HTTPBIN_URL,
        [["anything"], ["PUT", "GET", "POST", "DELETE"]],
        dict(
            method="FUZ2Z",
            filter='content~FUZ2Z and content~\'"var": "1"\' and content~\'"var2": "2"\'',
        ),
        [(200, "/anything")] * 4,
        None,
    ),
    (
        "test_static_strquery_set_multiple_method_gre",
        "%s/FUZZ?var=1&var2=2" % HTTPBIN_URL,
        [["anything"], ["PUT", "GET", "POST", "DELETE"]],
        dict(
            method="FUZ2Z",
            filter='content|gre(\'"method": "(.*)?",\')=FUZ2Z and content~\'"var": "1"\' and content~\'"var2": "2"\'',
        ),
        [(200, "/anything")] * 4,
        None,
    ),
    # set static HTTP values
    (
        "test_static_strquery_set",
        "%s:8000/FUZZ?var=1&var=2" % LOCAL_DOMAIN,
        [["echo"]],
        dict(filter="content=~'query=var=1&var=2$'"),
        [(200, "/echo")],
        None,
    ),
    (
        "test_static_postdata_set",
        "%s:8000/FUZZ" % LOCAL_DOMAIN,
        [["echo"]],
        dict(postdata="a=2", filter="content=~'POST_DATA=a=2$'"),
        [(200, "/echo")],
        None,
    ),
    (
        "test_static_postdata2_set",
        "%s:8000/FUZZ" % LOCAL_DOMAIN,
        [["echo"]],
        dict(postdata="2", filter="content=~'POST_DATA=2$'"),
        [(200, "/echo")],
        None,
    ),
    (
        "test_empty_postdata",
        "%s/FUZZ" % HTTPBIN_URL,
        [["anything"]],
        dict(
            postdata="",
            filter="content~'POST' and content~'\"form\": {},' and r.method='POST'",
        ),
        [(200, "/anything")],
        None,
    ),
    (
        "test_static_postdata3_set",
        "%s:8000/FUZZ" % LOCAL_DOMAIN,
        [["echo"]],
        dict(
            headers=[("Content-Type", "application/json")],
            postdata="2",
            filter="content=~'POST_DATA=2$' and content=~'command=POST$' and content~'Content-Type: application/json'",
        ),
        [(200, "/echo")],
        None,
    ),
    (
        "test_static_postdata3_set2",
        "%s:8000/FUZZ" % LOCAL_DOMAIN,
        [["echo"]],
        dict(
            headers=[("Content-Type", "aaaa")],
            postdata="a=2&b=3",
            filter="(content=~'POST_DATA=a=2&b=3$' or content=~'POST_DATA=b=3&a=2$') and content=~'command=POST$' and content~'Content-Type: aaaa'",
        ),
        [(200, "/echo")],
        None,
    ),
    (
        "test_static_postdata3_set3",
        "%s:8000/FUZZ" % LOCAL_DOMAIN,
        [["echo"]],
        dict(
            headers=[("Content-Type", "application/json")],
            postdata='{"a": "2"}',
            filter="content=~'POST_DATA={\"a\": \"2\"}$' and content=~'command=POST$' and content~'Content-Type: application/json'",
        ),
        [(200, "/echo")],
        None,
    ),
    (
        "test_static_method_set",
        "%s/FUZZ" % URL_LOCAL,
        [["dir"]],
        dict(
            method="OPTIONS",
            filter="content~'Message: Unsupported method (\\'OPTIONS\\')'",
        ),
        [(501, "/dir/dir")],
        None,
    ),
    (
        "test_static_header_set",
        "%s:8000/FUZZ" % LOCAL_DOMAIN,
        [["echo"]],
        dict(headers=[("myheader", "isset")], filter="content~'Myheader: isset'"),
        [(200, "/echo")],
        None,
    ),
    (
        "test_static_cookie_set",
        "%s:8000/FUZZ" % LOCAL_DOMAIN,
        [["echo"]],
        dict(cookie=["cookie1=value1"], filter="content~'Cookie: cookie1=value1'"),
        [(200, "/echo")],
        None,
    ),
    (
        "test_static_basic_auth_set",
        "%s:8000/FUZZ" % LOCAL_DOMAIN,
        [["echo"]],
        dict(
            auth={"method": "basic", "credentials": "user:pass"},
            filter="content~'Authorization: Basic dXNlcjpwYXNz'",
        ),
        [(200, "/echo")],
        None,
    ),
    (
        "test_static_ntlm_auth_set",
        "%s:8000/FUZZ" % LOCAL_DOMAIN,
        [["echo"]],
        dict(
            auth={"method": "ntlm", "credentials": "user:pass"},
            filter="content~'Authorization: NTLM TlRMTVNTUAABAAAABoIIAAAAAAAAAAAAAAAAAAAAAAA='",
        ),
        [(200, "/echo")],
        None,
    ),
    # fuzzing HTTP values
    (
        "test_basic_path_fuzz",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(),
        [(200, "/dir/a"), (200, "/dir/b"), (200, "/dir/c")],
        None,
    ),
    (
        "test_multi_path_fuzz",
        "%s/FUZZ/FUZ2Z/FUZ3Z" % ECHO_URL,
        [["a"], ["b"], ["c"]],
        dict(filter="content~'path=/echo/a/b/c'"),
        [(200, "/echo/a/b/c")],
        None,
    ),
    (
        "test_basic_method_fuzz",
        "%s" % URL_LOCAL,
        [["OPTIONS", "HEAD"]],
        dict(method="FUZZ", filter="content~'Unsupported method' and content~FUZZ"),
        [(501, "/dir")],
        None,
    ),
    (
        "test_basic_postdata_fuzz",
        "%s" % ECHO_URL,
        [["onevalue", "twovalue"]],
        dict(postdata="a=FUZZ", filter="content~FUZZ and content~'POST_DATA=a='"),
        [(200, "/echo"), (200, "/echo")],
        None,
    ),
    (
        "test_basic_postdata2_fuzz",
        "%s" % ECHO_URL,
        [["onevalue", "twovalue"]],
        dict(
            postdata="FUZZ=1234",
            filter="content~'POST_DATA=twovalue=1234' or content~'POST_DATA=onevalue=1234'",
        ),
        [(200, "/echo"), (200, "/echo")],
        None,
    ),
    (
        "test_basic_postdata3_fuzz",
        "%s" % ECHO_URL,
        [["onevalue", "twovalue"]],
        dict(
            postdata="FUZZ",
            filter="content~'POST_DATA=twovalue' or content~'POST_DATA=onevalue'",
        ),
        [(200, "/echo"), (200, "/echo")],
        None,
    ),
    (
        "test_basic_header_fuzz",
        "%s" % ECHO_URL,
        [["onevalue", "twovalue"]],
        dict(
            headers=[("myheader", "FUZZ")],
            filter="content~'Myheader:' and content~FUZZ",
        ),
        [(200, "/echo"), (200, "/echo")],
        None,
    ),
    (
        "test_basic_header_name_fuzz",
        "%s" % ECHO_URL,
        [["onevalue", "twovalue"]],
        dict(
            headers=[("FUZZ", "myheadervalue")],
            filter="content~': myheadervalue' and content~FUZZ",
        ),
        [(200, "/echo"), (200, "/echo")],
        None,
    ),
    (
        "test_static_strquery_fuzz",
        "%s:8000/echo?var=FUZZ" % LOCAL_DOMAIN,
        [["value1"]],
        dict(filter="content~'query=var=value1'"),
        [(200, "/echo")],
        None,
    ),
    (
        "test_static_strquery2_fuzz",
        "%s:8000/echo?FUZZ=value1" % LOCAL_DOMAIN,
        [["var"]],
        dict(filter="content~'query=var=value1'"),
        [(200, "/echo")],
        None,
    ),
    (
        "test_basic_cookie_fuzz",
        "%s/anything" % HTTPBIN_URL,
        [["cookievalue"]],
        dict(cookie=["test=FUZZ"], filter="content~FUZZ"),
        [(200, "/anything")],
        None,
    ),
    # url fuzzing
    (
        "test_url_with_no_path",
        "http://localhost:8000",
        [["GET"]],
        dict(method="FUZZ"),
        [(200, "/")],
        None,
    ),
    # travis uses old pycurl version ("test_url_not_normalized_by_lib", "http://localhost:8000/echo/FUZZ", [["../../etc/pass"]], dict(), [(200, '/echo/../../etc/pass')], None),
    (
        "test_url_port_fuzz",
        "%s:FUZZ/dir/a" % LOCAL_DOMAIN,
        [["8000"]],
        dict(),
        [(200, "/dir/a")],
        None,
    ),
    (
        "test_url_hostname_fuzz",
        "http://FUZZ:8000/dir/a",
        [["localhost"]],
        dict(),
        [(200, "/dir/a")],
        None,
    ),
    (
        "test_url_hostname2_fuzz",
        "http://FUZZ/dir/a",
        [["localhost:8000"]],
        dict(),
        [(200, "/dir/a")],
        None,
    ),
    (
        "test_url_schema_fuzz",
        "FUZZ://localhost:8000/dir/a",
        [["http"]],
        dict(),
        [(200, "/dir/a")],
        None,
    ),
    (
        "test_url_all_url_fuzz",
        "FUZZ",
        [["http://localhost:8000/dir/a"]],
        dict(),
        [(200, "/dir/a")],
        None,
    ),
    (
        "test_url_all_url_fuzz2",
        "FUZZ",
        [["%s/anything/datastore/search_get_by_name.php?name=Rake" % HTTPBIN_URL]],
        dict(),
        [(200, "/anything/datastore/search_get_by_name.php")],
        None,
    ),
    # edge cases
    (
        "test_vhost_fuzz",
        "%s" % ECHO_URL,
        [["onevalue", "twovalue"]],
        dict(headers=[("Host", "FUZZ")], filter="content~'Host:' and content~FUZZ"),
        [(200, "/echo"), (200, "/echo")],
        None,
    ),
    # payload encoder tests
    (
        "test_encoding",
        "%s:8000/echo?var=FUZZ" % LOCAL_DOMAIN,
        None,
        dict(
            payloads=[("list", dict(values="value1", encoder=["md5"]))],
            filter="content~'path=/echo?var=9946687e5fa0dab5993ededddb398d2e'",
        ),
        [(200, "/echo")],
        None,
    ),
    (
        "test_nested_encoding",
        "%s:8000/echo?var=FUZZ" % LOCAL_DOMAIN,
        None,
        dict(
            payloads=[("list", dict(values="value1", encoder=["none@md5"]))],
            filter="content~'path=/echo?var=9946687e5fa0dab5993ededddb398d2e'",
        ),
        [(200, "/echo")],
        None,
    ),
    (
        "test_cat_encoding",
        "%s:8000/echo?var=FUZZ" % LOCAL_DOMAIN,
        None,
        dict(
            payloads=[("list", dict(values="value1", encoder=["default"]))],
            filter="content~'path=/echo?var=' and (content~'9946687e5fa0dab5993ededddb398d2e' or content~'value1')",
        ),
        [(200, "/echo"), (200, "/echo")],
        None,
    ),
    # prefilter, slice
    (
        "test_prefilter",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "a", "a", "a", "a", "a"]],
        dict(prefilter=["FUZZ|u()"], ss="one"),
        [(200, "/dir/a")],
        None,
    ),
    (
        "test_slice",
        "%s/FUZZ" % URL_LOCAL,
        None,
        dict(payloads=[("list", dict(default="a-a-a-a-a"), "FUZZ|u()")], ss="one"),
        [(200, "/dir/a")],
        None,
    ),
    (
        "test_slice2",
        "%s/FUZZ" % URL_LOCAL,
        None,
        dict(payloads=[("range", dict(default="1-10"), "FUZZ='1'")]),
        [(404, "/dir/1")],
        None,
    ),
    (
        "test_slice_rw",
        "%s/FUZZ" % URL_LOCAL,
        None,
        dict(payloads=[("range", dict(default="1-3"), "'1'")]),
        [(404, "/dir/1"), (404, "/dir/1"), (404, "/dir/1")],
        None,
    ),
    (
        "test_slice_rw_int",
        "%s/FUZZ" % URL_LOCAL,
        None,
        dict(payloads=[("range", dict(default="1-3"), "1")]),
        [(404, "/dir/1"), (404, "/dir/1"), (404, "/dir/1")],
        None,
    ),
    (
        "test_slice_rw_upper_int",
        "%s/FUZZ" % URL_LOCAL,
        None,
        dict(payloads=[("range", dict(default="1-3"), "FUZZ|upper()")]),
        [(404, "/dir/1"), (404, "/dir/2"), (404, "/dir/3")],
        None,
    ),
    (
        "test_slice_rw_upper_int",
        "%s/FUZZ" % URL_LOCAL,
        None,
        dict(payloads=[("range", dict(default="1-3"), "FUZZ|upper()")]),
        [(404, "/dir/1"), (404, "/dir/2"), (404, "/dir/3")],
        None,
    ),
    (
        "test_slice_rw_replace_int",
        "%s/FUZZ" % URL_LOCAL,
        None,
        dict(payloads=[("range", dict(default="1-3"), "FUZZ|replace(1,'one')")]),
        [(200, "/dir/one"), (404, "/dir/2"), (404, "/dir/3")],
        None,
    ),
    (
        "test_slice_rw_replace_int_2",
        "%s/FUZZ" % URL_LOCAL,
        None,
        dict(payloads=[("range", dict(default="1-3"), "FUZZ|replace('1','one')")]),
        [(200, "/dir/one"), (404, "/dir/2"), (404, "/dir/3")],
        None,
    ),
    (
        "test_slice_rw_replace_int_3",
        "%s/FUZZ" % URL_LOCAL,
        None,
        dict(
            payloads=[("list", dict(default="one-two"), "FUZZ|replace('one','three')")]
        ),
        [(404, "/dir/two"), (404, "/dir/three")],
        None,
    ),
    # follow
    (
        "test_follow",
        "%s:8000/FUZZ" % LOCAL_DOMAIN,
        [["redirect"]],
        dict(follow=True, filter="content~'path=/echo'"),
        [(200, "/echo")],
        None,
    ),
    # all params
    (
        "test_all_params_get",
        "%s:8000/echo?var=1&var2=2" % LOCAL_DOMAIN,
        [["avalue"]],
        dict(
            allvars="allvars",
            filter="content~'query=var=avalue&var2=2' or content~'var=1&var2=avalue'",
        ),
        [(200, "/echo"), (200, "/echo")],
        None,
    ),
    (
        "test_all_params_post",
        "%s" % ECHO_URL,
        [["onevalue"]],
        dict(
            allvars="allpost",
            postdata="a=1&b=2",
            filter="content~'command=POST' and (content~'a=onevalue' and content~'b=2') or (content~'a=1' and content~'b=onevalue')",
        ),
        [(200, "/echo"), (200, "/echo")],
        None,
    ),
    # simple filter
    (
        "test_codes_HC",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(hc=[404]),
        [(200, "/dir/a"), (200, "/dir/b"), (200, "/dir/c")],
        None,
    ),
    (
        "test_codes_SC",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(sc=[200]),
        [(200, "/dir/a"), (200, "/dir/b"), (200, "/dir/c")],
        None,
    ),
    (
        "test_codes_HL",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(hl=[4]),
        [(200, "/dir/b")],
        None,
    ),
    (
        "test_codes_SL",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(sl=[4]),
        [(200, "/dir/a"), (200, "/dir/c")],
        None,
    ),
    (
        "test_codes_HW",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(hw=[11]),
        [(200, "/dir/a"), (200, "/dir/b")],
        None,
    ),
    (
        "test_codes_SW",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(sw=[11]),
        [(200, "/dir/c")],
        None,
    ),
    (
        "test_codes_HH",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(hh=[28]),
        [(200, "/dir/b"), (200, "/dir/c")],
        None,
    ),
    (
        "test_codes_SH",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(sh=[28]),
        [(200, "/dir/a")],
        None,
    ),
    # combining simple filters
    (
        "test_hchlhhhw",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c", "d", "e", "f"]],
        dict(hc=[404], hl=[4], hh=[300]),
        [(200, "/dir/b")],
        None,
    ),
    (
        "test_shsw",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c", "d", "e", "f"]],
        dict(sh=[28], sw=[6]),
        [(200, "/dir/a")],
        None,
    ),
    # regex filter
    (
        "test_ss",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c", "d", "e", "f"]],
        dict(ss="one"),
        [(200, "/dir/a"), (200, "/dir/b")],
        None,
    ),
    (
        "test_hs",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(hs="one"),
        [(200, "/dir/c")],
        None,
    ),
    (
        "test_regex_sc",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c", "d", "e", "f"]],
        dict(sc=[200], ss="one"),
        [(200, "/dir/a"), (200, "/dir/b")],
        None,
    ),
    (
        "test_regex_hc",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c", "d", "e", "f"]],
        dict(hc=[200], ss="one"),
        [],
        None,
    ),
    # complex filter
    (
        "test_filter_clh",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(filter="c!=404 and l!=4 and h!=300 and w!=6"),
        [(200, "/dir/b")],
        None,
    ),
    (
        "test_filter_hw",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(filter="h=28 or w=6"),
        [(200, "/dir/a")],
        None,
    ),
    (
        "test_combined_filter",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(filter="h=28", sw=[6]),
        [(200, "/dir/a")],
        None,
    ),
    (
        "test_filter_intext",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(filter="content~'one'"),
        [(200, "/dir/a"), (200, "/dir/b")],
        None,
    ),
    (
        "test_filter_intext2",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(filter="content!~'one'"),
        [(200, "/dir/c")],
        None,
    ),
    (
        "test_dict_filter_strquery_fuzz",
        "%s:8000/echo?var=FUZZ" % LOCAL_DOMAIN,
        [["value1"]],
        dict(filter="r.params.get~'value1'"),
        [(200, "/echo")],
        None,
    ),
    # baseline
    (
        "test_baseline_header",
        "%s" % ECHO_URL,
        [["twovalue"]],
        dict(
            headers=[("FUZZ{onevalue}", "admin")],
            filter="(content~'onevalue:' or content~'twovalue:') and content~'admin'",
        ),
        [(200, "/echo"), (200, "/echo")],
        None,
    ),
    (
        "test_baseline_header_content",
        "%s" % ECHO_URL,
        [["twovalue"]],
        dict(
            headers=[("myheader", "FUZZ{onevalue}")],
            filter="content~'Myheader:' and (content~FUZZ or content~BBB)",
        ),
        [(200, "/echo"), (200, "/echo")],
        None,
    ),
    (
        "test_baseline",
        "%s/FUZZ{notthere}" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(),
        [(200, "/dir/a"), (200, "/dir/b"), (200, "/dir/c"), (404, "/dir/notthere")],
        None,
    ),
    (
        "test_baseline2",
        "%s/FUZZ{notthere}" % URL_LOCAL,
        [["a", "b", "c", "d", "e", "f"]],
        dict(hc=["BBB"]),
        [(200, "/dir/a"), (200, "/dir/b"), (200, "/dir/c")] + [(404, "/dir/notthere")],
        None,
    ),
    (
        "test_baseline_filter",
        "%s/FUZZ{notthere}" % URL_LOCAL,
        [["a", "b", "c", "d", "e", "f"]],
        dict(filter="c!=BBB"),
        [(200, "/dir/a"), (200, "/dir/b"), (200, "/dir/c")] + [(404, "/dir/notthere")],
        None,
    ),
    (
        "test_baseline3",
        "%s/FUZZ{notthere}" % URL_LOCAL,
        [["a", "b", "c"]],
        dict(hc=[200]),
        [(404, "/dir/notthere")],
        None,
    ),
    # XXX("test_scheme_baseline_fuzz", "FUZZ{HTTP}://localhost:8000/dir/a", [["https"]], dict(), [(200, '/dir/a')], None),
    # iterators
    (
        "test_product",
        "%s:8000/iterators/FUZZFUZ2Z" % LOCAL_DOMAIN,
        [["a", "b"], ["c"]],
        dict(iterator="product"),
        [(200, "/iterators/ac"), (404, "/iterators/bc")],
        None,
    ),
    (
        "test_zip",
        "%s:8000/iterators/FUZZFUZ2Z" % LOCAL_DOMAIN,
        [["a", "b"], ["c"]],
        dict(iterator="zip"),
        [(200, "/iterators/ac")],
        None,
    ),
    (
        "test_chain",
        "%s/FUZZ" % URL_LOCAL,
        [["a", "b"], ["c"]],
        dict(iterator="chain"),
        [(200, "/dir/a"), (200, "/dir/b"), (200, "/dir/c")],
        None,
    ),
    # recursive
    (
        "test_rlevel_1",
        "%s:8000/recursive_dir/FUZZ" % LOCAL_DOMAIN,
        [["a", "b", "c"]],
        dict(sc=[301], rlevel=1),
        [(301, "/recursive_dir/a"), (301, "/recursive_dir/a/b")],
        None,
    ),
    (
        "test_rlevel_2",
        "%s:8000/recursive_dir/FUZZ" % LOCAL_DOMAIN,
        [["a", "b", "c"]],
        dict(sc=[301], rlevel=2),
        [
            (301, "/recursive_dir/a"),
            (301, "/recursive_dir/a/b"),
            (301, "/recursive_dir/a/b/c"),
        ],
        None,
    ),
    (
        "test_rlevel_1_post",
        "%s:8000/echo/FUZZ/" % LOCAL_DOMAIN,
        [["a"]],
        dict(
            filter="content~'command=POST' and content~'POST_DATA=a=1'",
            postdata="a=1",
            rlevel=1,
        ),
        [(200, "/echo/a/"), (200, "/echo/a/a")],
        None,
    ),
    # plugins
    (
        "test_robots",
        "%s:8000/plugins/FUZZ" % LOCAL_DOMAIN,
        [["robots.txt"]],
        dict(script="robots"),
        [
            (404, "/cal_endar/"),
            (404, "/crawlsnags/"),
            (404, "/osrun/"),
            (200, "/plugins/robots.txt"),
            (200, "/static/"),
        ],
        None,
    ),
    (
        "test_robots_hc",
        "%s:8000/plugins/FUZZ" % LOCAL_DOMAIN,
        [["robots.txt"]],
        dict(hc=[404], script="robots"),
        [(200, "/plugins/robots.txt"), (200, "/static/")],
        None,
    ),
    (
        "test_plugins_filter",
        "%s/FUZZ" % HTTPBIN_URL,
        [["anything"]],
        dict(script="headers", filter="plugins~'unicorn'"),
        [(200, "/anything")],
        None,
    ),
]

scanmode_tests = [
    (
        "test_scanmode",
        "%s:666/FUZZ" % LOCAL_DOMAIN,
        [["a", "b", "c"]],
        dict(scanmode=True),
        [(-1, "/a"), (-1, "/b"), (-1, "/c")],
        None,
    ),
    (
        "test_scanmode_sc",
        "%s:666/FUZZ" % LOCAL_DOMAIN,
        [["a", "b", "c"]],
        dict(scanmode=True, sc=[-1]),
        [(-1, "/a"), (-1, "/b"), (-1, "/c")],
        None,
    ),
    (
        "test_scanmode_sc_xxx",
        "%s:666/FUZZ" % LOCAL_DOMAIN,
        [["a", "b", "c"]],
        dict(scanmode=True, sc=["XXX"]),
        [(-1, "/a"), (-1, "/b"), (-1, "/c")],
        None,
    ),
    (
        "test_scanmode_hc",
        "%s:666/FUZZ" % LOCAL_DOMAIN,
        [["a", "b", "c"]],
        dict(scanmode=True, hc=[-1]),
        [],
        None,
    ),
    (
        "test_scanmode_hc_xxx",
        "%s:666/FUZZ" % LOCAL_DOMAIN,
        [["a", "b", "c"]],
        dict(scanmode=True, hc=["XXX"]),
        [],
        None,
    ),
]

error_tests = [
    (
        "test_url_schema_error_fuzz",
        "FUZZ://localhost:8000/dir/a",
        [["https"]],
        dict(),
        [(200, "/dir/a")],
        "Pycurl error 35",
    ),
    (
        "test_all_params_fuzz_error",
        "%s:8000/echo?var=FUZZ&var2=2" % LOCAL_DOMAIN,
        [["avalue"]],
        dict(
            allvars="allvars",
            filter="content~'query=var=avalue&var2=2' or content~'var=1&var2=avalue'",
        ),
        [(200, "/echo"), (200, "/echo")],
        "FUZZ words not allowed when using all parameters brute forcing",
    ),
    (
        "test_all_params_no_var",
        "%s:8000/echo" % LOCAL_DOMAIN,
        [["avalue"]],
        dict(
            allvars="allvars",
            filter="content~'query=var=avalue&var2=2' or content~'var=1&var2=avalue'",
        ),
        [(200, "/echo"), (200, "/echo")],
        "No variables on specified variable set",
    ),
    (
        "test_bad_port",
        "%s:6666/FUZZ" % LOCAL_DOMAIN,
        [list(range(1))],
        dict(),
        [],
        "Failed to connect to localhost port 6666",
    ),
    (
        "test_bad_num_payloads",
        "%s:8000/FUZZ" % LOCAL_DOMAIN,
        [list(range(1)), list(range(1))],
        dict(),
        [],
        "FUZZ words and number of payloads do not match",
    ),
    (
        "test_bad_proxy",
        "%s:8000/FUZZ" % LOCAL_DOMAIN,
        [list(range(1))],
        dict(proxies=[("localhost", 888, "HTTP")]),
        [],
        "Failed to connect to localhost port 888",
    ),
    (
        "test_bad_num_dic",
        "%s:8000/iterators/FUZZ" % LOCAL_DOMAIN,
        [list(range(1))],
        dict(iterator="zip"),
        [],
        "Several dictionaries must be used when specifying an iterator",
    ),
]


class DynamicTests(unittest.TestCase):
    """
    Dummy class that will be populated dynamically with all the tests
    """

    pass


def wfuzz_me_test_generator(url, payloads, params, expected_list, extra_params):
    def test(self):
        # Wfuzz results
        with wfuzz.FuzzSession(url=url, **params) as s:
            if payloads is None:
                fuzzed = s.fuzz()
            else:
                fuzzed = s.get_payloads(payloads).fuzz()

            ret_list = [(x.code, x.history.urlparse.path) for x in fuzzed]

        # repeat test with extra params if specified and check against
        if extra_params:
            # if using proxy change localhost for docker compose service
            proxied_url = url
            proxied_payloads = payloads
            if "proxies" in extra_params:
                for original_host, proxied_host in REPLACE_HOSTNAMES:
                    proxied_url = proxied_url.replace(original_host, proxied_host)
                    if proxied_payloads:
                        proxied_payloads = [
                            [
                                payload.replace(original_host, proxied_host)
                                for payload in payloads_list
                            ]
                            for payloads_list in proxied_payloads
                        ]

                if "connect_to_ip" in extra_params and extra_params["connect_to_ip"]:
                    extra_params["connect_to_ip"]["ip"] = "httpbin"
                    extra_params["connect_to_ip"]["port"] = "80"

            with wfuzz.FuzzSession(url=proxied_url) as s:
                same_list = [
                    (x.code, x.history.urlparse.path)
                    for x in s.get_payloads(proxied_payloads).fuzz(**extra_params)
                ]

            self.assertEqual(sorted(ret_list), sorted(same_list))
        else:
            self.assertEqual(sorted(ret_list), sorted(expected_list))

    return test


def wfuzz_me_test_generator_exception(fn, exception_string):
    def test_exception(self):
        with self.assertRaises(Exception) as context:
            fn(None)
            self.assertTrue(exception_string in str(context.exception))

    return test_exception


def wfuzz_me_test_generator_saveres(url, payloads, params, expected_list):
    def test(self):
        if not expected_list:
            return
        temp_name = next(tempfile._get_candidate_names())
        defult_tmp_dir = tempfile._get_default_tempdir()

        filename = os.path.join(defult_tmp_dir, temp_name)

        # Wfuzz results
        with wfuzz.FuzzSession(
            url=url, **dict(list(params.items()) + list(dict(save=filename).items()))
        ) as s:
            if payloads is None:
                fuzzed = s.fuzz()
            else:
                fuzzed = s.get_payloads(payloads).fuzz()

            ret_list = [(x.code, x.history.urlparse.path) for x in fuzzed]

        # repeat test with performaing same saved request
        with wfuzz.FuzzSession(
            payloads=[("wfuzzp", dict(fn=filename))], url="FUZZ"
        ) as s:
            same_list = [(x.code, x.history.urlparse.path) for x in s.fuzz()]

        self.assertEqual(sorted(ret_list), sorted(same_list))

        # repeat test with performaing FUZZ[url] saved request
        with wfuzz.FuzzSession(
            payloads=[("wfuzzp", dict(fn=filename))], url="FUZZ[url]"
        ) as s:
            same_list = [(x.code, x.history.urlparse.path) for x in s.fuzz()]

        self.assertEqual(sorted(ret_list), sorted(same_list))

    return test


def wfuzz_me_test_generator_recipe(url, payloads, params, expected_list):
    def test(self):
        temp_name = next(tempfile._get_candidate_names())
        defult_tmp_dir = tempfile._get_default_tempdir()

        filename = os.path.join(defult_tmp_dir, temp_name)

        # Wfuzz results
        with wfuzz.FuzzSession(url=url, **params) as s:
            s.export_to_file(filename)

            if payloads is None:
                fuzzed = s.fuzz()
            else:
                fuzzed = s.get_payloads(payloads).fuzz()

            ret_list = [(x.code, x.history.urlparse.path) for x in fuzzed]

        # repeat test with recipe as only parameter
        with wfuzz.FuzzSession(recipe=[filename]) as s:
            if payloads is None:
                same_list = [(x.code, x.history.urlparse.path) for x in s.fuzz()]
            else:
                same_list = [
                    (x.code, x.history.urlparse.path)
                    for x in s.get_payloads(payloads).fuzz()
                ]

        self.assertEqual(sorted(ret_list), sorted(same_list))

    return test


def wfuzz_me_test_generator_previous_session(
    prev_session_cli, next_session_cli, expected_list
):
    def test(self):
        temp_name = next(tempfile._get_candidate_names())
        defult_tmp_dir = tempfile._get_default_tempdir()

        filename = os.path.join(defult_tmp_dir, temp_name)

        # first session
        with wfuzz.get_session(prev_session_cli) as s:
            ret_list = [
                x._field() if x._fields else x.description
                for x in s.fuzz(save=filename)
            ]

        # second session wfuzzp as payload
        with wfuzz.get_session(next_session_cli.replace("$$PREVFILE$$", filename)) as s:
            ret_list = [x._field() if x._fields else x.description for x in s.fuzz()]

        self.assertEqual(sorted(ret_list), sorted(expected_list))

    return test


def create_test(
    test_name, url, payloads, params, expected_res, extra_params, exception_str
):
    test_fn = wfuzz_me_test_generator(url, payloads, params, expected_res, extra_params)
    if exception_str:
        test_fn_exc = wfuzz_me_test_generator_exception(test_fn, exception_str)
        setattr(DynamicTests, test_name, test_fn_exc)
    else:
        setattr(DynamicTests, test_name, test_fn)


def create_tests_from_list(test_list):
    """
    Creates tests cases where wfuzz using the indicated url, params results are checked against expected_res
    """
    for test_name, url, payloads, params, expected_res, exception_str in test_list:
        create_test(test_name, url, payloads, params, expected_res, None, exception_str)


def duplicate_tests_diff_params(
    test_list, group, next_extra_params, previous_extra_params
):
    """
    Ignores expected_res and generates wfuzz tests that run 2 times with different params, expecting same results.

    """
    for test_name, url, payloads, params, expected_res, exception_str in test_list:

        # mitmproxy does not go well with encodings. temporary bypass encoding checks with proxy
        if group == "_proxy_" and "encode" in test_name:
            continue

        next_extra = copy.deepcopy(params)
        next_extra.update(next_extra_params)
        new_test = "%s_%s" % (test_name, group)

        prev_extra = copy.deepcopy(params)
        if previous_extra_params:
            prev_extra.update(previous_extra_params)

        create_test(
            new_test, url, payloads, prev_extra, None, next_extra, exception_str
        )


def duplicate_tests(test_list, group, test_gen_fun):
    """
    generates wfuzz tests that run 2 times with recipe input, expecting same results.

    """
    for test_name, url, payloads, params, expected_res, exception_str in test_list:
        new_test = "%s_%s" % (test_name, group)

        test_fn = test_gen_fun(url, payloads, params, None)
        if exception_str:
            test_fn_exc = wfuzz_me_test_generator_exception(test_fn, exception_str)
            setattr(DynamicTests, new_test, test_fn_exc)
        else:
            setattr(DynamicTests, new_test, test_fn)


def create_savedsession_tests(test_list, test_gen_fun):
    """
    generates wfuzz tests that run 2 times with a saved session, expecting same results.

    """
    for test_name, prev_cli, next_cli, expected_res, exception_str in test_list:
        test_fn = test_gen_fun(prev_cli, next_cli, expected_res)
        if exception_str:
            test_fn_exc = wfuzz_me_test_generator_exception(test_fn, exception_str)
            setattr(DynamicTests, test_name, test_fn_exc)
        else:
            setattr(DynamicTests, test_name, test_fn)


def create_tests():
    """
    Creates all dynamic tests

    """
    if testing_savedsession_tests:
        create_savedsession_tests(
            testing_savedsession_tests, wfuzz_me_test_generator_previous_session
        )
        return

    if testing_tests:
        create_tests_from_list(testing_tests)
        duplicate_tests(testing_tests, "recipe", wfuzz_me_test_generator_recipe)
        duplicate_tests(testing_tests, "saveres", wfuzz_me_test_generator_saveres)
        duplicate_tests_diff_params(
            testing_tests, "_proxy_", dict(proxies=[("localhost", 8080, "HTTP")]), None
        )
    else:
        # this are the basics
        basic_functioning_tests = [error_tests, scanmode_tests, basic_tests]

        for t in basic_functioning_tests:
            create_tests_from_list(t)

        # description tests
        create_savedsession_tests(
            savedsession_tests, wfuzz_me_test_generator_previous_session
        )

        # duplicate tests with recipe
        duplicate_tests(basic_tests, "recipe", wfuzz_me_test_generator_recipe)

        # duplicate tests with save results
        duplicate_tests(basic_tests, "saveres", wfuzz_me_test_generator_saveres)

        # duplicate tests with proxy
        duplicate_tests_diff_params(
            basic_tests, "_proxy_", dict(proxies=[("localhost", 8080, "HTTP")]), None
        )


create_tests()

if __name__ == "__main__":
    unittest.main()



import pytest


@pytest.mark.parametrize(
    "filter_string, expected_result",
    [("r.code:=429", 429), ("r.c:=404", 404), ("r.c=+404", 706), ("r.c=-2", 300)],
)
def test_code_set(filter_obj, example_full_fuzzres, filter_string, expected_result):
    filter_obj.is_visible(example_full_fuzzres, filter_string)
    assert example_full_fuzzres.code == expected_result



import sys


try:
    # Python >= 3.3
    from unittest import mock
except ImportError:
    # Python < 3.3
    import mock

import unittest

from wfuzz.externals.moduleman.loader import DirLoader
from wfuzz.externals.moduleman.loader import FileLoader
from wfuzz.externals.moduleman.registrant import BRegistrant
from wfuzz.externals.moduleman.plugin import moduleman_plugin
import wfuzz.externals.moduleman.modulefilter as modulefilter


@moduleman_plugin
class test_plugin1:
    name = "test_plugin1"
    author = (("test plugin 1"),)
    version = "0.1"
    description = ("test plugin 1",)
    summary = "test plugin 1."
    category = ["aggressive"]
    priority = 79

    parameters = (("test", "", True, "test plugin 1"),)


@moduleman_plugin
class test_plugin2:
    name = "test_plugin2"
    author = (("test plugin 2"),)
    version = "0.1"
    description = ("test plugin 2",)
    summary = "test plugin 2."
    category = ["default"]
    priority = 89

    parameters = (("test", "", True, "test plugin 2"),)


@moduleman_plugin
class test_plugin3:
    name = "test_plugin3"
    author = (("test plugin 3"),)
    version = "0.1"
    description = ("test plugin 3",)
    summary = "test plugin 3."
    category = ["safe", "default"]
    priority = 99

    parameters = (("test", "", True, "test plugin 3"),)


class ModuleFilterTests(unittest.TestCase):
    def test_load_dir2(self):
        with mock.patch("os.listdir") as mocked_listdir:
            with mock.patch("os.path.isdir") as mocked_isdir:
                with mock.patch("os.path.isfile") as mocked_isfile:
                    with mock.patch("imp.find_module") as mocked_find_module:
                        with mock.patch("imp.load_module") as mocked_load_module:
                            mocked_listdir.return_value = ["alpha", "project.py"]
                            mocked_isdir.side_effect = [True, False]
                            mocked_isfile.return_value = True
                            mocked_find_module.return_value = (
                                None,
                                "/any/alpha/project.py",
                                (".py", "U", 1),
                            )
                            mocked_load_module.return_value = sys.modules[__name__]

                            br = BRegistrant(
                                DirLoader(**{"base_dir": "beta", "base_path": "any"})
                            )

                            self.assertEqual(
                                sorted(br.get_plugins_names()),
                                sorted(
                                    ["test_plugin1", "test_plugin2", "test_plugin3"]
                                ),
                            )
                            self.assertEqual(
                                br.get_plugins_names("default"),
                                ["test_plugin2", "test_plugin3"],
                            )
                            self.assertEqual(
                                br.get_plugins_names("aggressive"), ["test_plugin1"]
                            )
                            self.assertEqual(
                                sorted(br.get_plugins_names("not aggressive")),
                                sorted(["test_plugin2", "test_plugin3"]),
                            )
                            self.assertEqual(
                                sorted(br.get_plugins_names("default or aggressive")),
                                sorted(
                                    ["test_plugin1", "test_plugin2", "test_plugin3"]
                                ),
                            )
                            self.assertEqual(
                                sorted(br.get_plugins_names("default and safe")),
                                sorted(["test_plugin3"]),
                            )
                            self.assertEqual(
                                sorted(br.get_plugins_names("test_pl*")),
                                sorted(
                                    ["test_plugin1", "test_plugin2", "test_plugin3"]
                                ),
                            )
                            self.assertEqual(
                                sorted(br.get_plugins_names("test_plugin1")),
                                sorted(["test_plugin1"]),
                            )

    def test_load_file(self):
        with mock.patch("imp.find_module") as mocked_find_module:
            with mock.patch("imp.load_module") as mocked_load_module:
                mocked_find_module.return_value = (
                    None,
                    "any/project.py",
                    (".py", "U", 1),
                )
                mocked_load_module.return_value = sys.modules[__name__]

                br = BRegistrant(
                    FileLoader(**{"filename": "project1.py", "base_path": "any"})
                )

                self.assertEqual(
                    sorted(br.get_plugins_names()),
                    sorted(["test_plugin1", "test_plugin2", "test_plugin3"]),
                )

                self.assertTrue(br.get_plugin("test_plugin1").name == "test_plugin1")
                self.assertTrue(br.get_plugin("test_plugin2").name == "test_plugin2")

                with self.assertRaises(Exception) as context:
                    br.get_plugin("test_")
                self.assertTrue("Multiple plugins found" in str(context.exception))

    def test_simple_filter(self):
        with mock.patch("imp.find_module") as mocked_find_module:
            with mock.patch("imp.load_module") as mocked_load_module:
                mocked_find_module.return_value = (
                    None,
                    "/any/project.py",
                    (".py", "U", 1),
                )
                mocked_load_module.return_value = sys.modules[__name__]

                br = BRegistrant(
                    FileLoader(**{"filename": "project1.py", "base_path": "any"})
                )

                with self.assertRaises(Exception) as context:
                    modulefilter.PYPARSING = False
                    br.get_plugins_names("not aggressive")
                self.assertTrue(
                    "Pyparsing missing, complex filters not allowed."
                    in str(context.exception)
                )

                modulefilter.PYPARSING = False
                self.assertEqual(
                    sorted(br.get_plugins_names("test*")),
                    sorted(["test_plugin1", "test_plugin2", "test_plugin3"]),
                )
                self.assertEqual(
                    sorted(br.get_plugins_names("test_plugin1,test_plugin2")),
                    sorted(["test_plugin1", "test_plugin2"]),
                )
                self.assertEqual(
                    sorted(br.get_plugins_names("test_plugin5")), sorted([])
                )

    def test_plugin_decorator(self):
        with self.assertRaises(Exception) as context:

            @moduleman_plugin("method1")
            class test_plugin4:
                pass

            test_plugin4()
            self.assertTrue(
                "Required method method4 not implemented" in str(context.exception)
            )



import unittest
import sys
from io import BytesIO
import gzip
import pickle as pickle

import wfuzz
from wfuzz.facade import Facade
from wfuzz.fuzzobjects import FuzzResult
from wfuzz.fuzzrequest import FuzzRequest

try:
    # Python >= 3.3
    from unittest import mock
except ImportError:
    # Python < 3.3
    import mock


class APITests(unittest.TestCase):
    def test_payload_description(self):
        class mock_saved_session(object):
            def __init__(self, fields, show_field):
                fr = FuzzRequest()
                fr.url = "http://www.wfuzz.org/path?param=1&param2=2"
                fuzz_res = FuzzResult(history=fr)
                fuzz_res._fields = fields
                fuzz_res._show_field = show_field

                self.outfile = BytesIO()

                with gzip.GzipFile(fileobj=self.outfile, mode="wb") as f:
                    pickle.dump(fuzz_res, f)

                self.outfile.seek(0)
                self.outfile.name = "mockfile"

            def close(self):
                pass

            def read(self, *args, **kwargs):
                return self.outfile.read(*args, **kwargs)

            def seek(self, *args, **kwargs):
                return self.outfile.seek(*args, **kwargs)

            def tell(self):
                return self.outfile.tell()

        # load plugins before mocking file object
        Facade().payloads

        m = mock.MagicMock(name="open", spec=open)
        m.return_value = mock_saved_session(["r.params.all"], True)

        mocked_fun = (
            "builtins.open" if sys.version_info >= (3, 0) else "__builtin__.open"
        )
        with mock.patch(mocked_fun, m):
            payload_list = list(
                wfuzz.payload(
                    **{
                        "show_field": True,
                        "fields": ["r"],
                        "payloads": [
                            ("wfuzzp", {"default": "mockedfile", "encoder": None}, None)
                        ],
                    }
                )
            )
            self.assertEqual(
                sorted(
                    "-".join([res[0].description for res in payload_list]).split("\n")
                ),
                sorted(["param=1", "param2=2"]),
            )

        m = mock.MagicMock(name="open", spec=open)
        m.return_value = mock_saved_session(["url"], None)

        mocked_fun = (
            "builtins.open" if sys.version_info >= (3, 0) else "__builtin__.open"
        )
        with mock.patch(mocked_fun, m):
            payload_list = list(
                wfuzz.payload(
                    **{
                        "show_field": True,
                        "fields": ["r"],
                        "payloads": [
                            ("wfuzzp", {"default": "mockedfile", "encoder": None}, None)
                        ],
                    }
                )
            )
            self.assertEqual(
                [res[0].description for res in payload_list],
                ["http://www.wfuzz.org/path?param=1&param2=2"],
            )

        m = mock.MagicMock(name="open", spec=open)
        m.return_value = mock_saved_session(["r.scheme"], False)

        mocked_fun = (
            "builtins.open" if sys.version_info >= (3, 0) else "__builtin__.open"
        )
        with mock.patch(mocked_fun, m):
            payload_list = list(
                wfuzz.payload(
                    **{
                        "show_field": True,
                        "fields": ["r"],
                        "payloads": [
                            ("wfuzzp", {"default": "mockedfile", "encoder": None}, None)
                        ],
                    }
                )
            )
            self.assertEqual(
                [res[0].description for res in payload_list],
                ["http://www.wfuzz.org/path?param=1&param2=2 | http"],
            )

    def test_payload(self):
        with mock.patch("os.walk") as mocked_oswalk:
            mocked_oswalk.return_value = [
                ("foo", ("bar",), ("baz",)),
                ("foo/bar", (), ("spam", "eggs")),
            ]
            payload_list = list(
                wfuzz.payload(
                    **{
                        "payloads": [
                            ("dirwalk", {"default": "foo", "encoder": None}, None)
                        ]
                    }
                )
            )
            self.assertEqual(
                sorted(payload_list), sorted([("baz",), ("bar/spam",), ("bar/eggs",)])
            )

        class mock_file(object):
            def __init__(self):
                self.my_iter = iter([b"one", b"two"])

            def __iter__(self):
                return self

            def __next__(self):
                return next(self.my_iter)

            def seek(self, pos):
                self.my_iter = iter([b"one", b"two"])

            next = __next__  # for Python 2

        m = mock.MagicMock(name="open", spec=open)
        m.return_value = mock_file()

        mocked_fun = (
            "builtins.open" if sys.version_info >= (3, 0) else "__builtin__.open"
        )
        with mock.patch(mocked_fun, m):
            payload_list = list(
                wfuzz.payload(
                    **{
                        "payloads": [
                            ("file", {"default": "mockedfile", "encoder": None}, None)
                        ]
                    }
                )
            )
            self.assertEqual(sorted(payload_list), sorted([("one",), ("two",)]))



import unittest

# Python 2 and 3: urlib.parse

from wfuzz.fuzzrequest import FuzzRequest
from wfuzz.ui.console.clparser import CLParser
from wfuzz import __version__ as wfuzz_version


raw_req = """GET /a HTTP/1.1
Host: www.wfuzz.org
Content-Type: application/x-www-form-urlencoded
User-Agent: Wfuzz/{}

""".format(
    wfuzz_version
)

raw_response_header = b"""HTTP/1.0 200 Connection established

HTTP/1.1 404 Not Found
Content-Type: text/html; charset=UTF-8
Referrer-Policy: no-referrer
Content-Length: 1564
Date: Wed, 24 Apr 2019 22:03:52 GMT
Alt-Svc: quic=":443"; ma=2592000; v="46,44,43,39"
Connection: close

"""

raw_response_body = b'<!DOCTYPE html>\n<html lang=en>\n  <meta charset=utf-8>\n  <meta name=viewport content="initial-scale=1, minimum-scale=1, width=device-width">\n  <title>Error 404 (Not Found)!!1</title>\n  <style>\n    *{margin:0;padding:0}html,code{font:15px/22px arial,sans-serif}html{background:#fff;color:#222;padding:15px}body{margin:7% auto 0;max-width:390px;min-height:180px;padding:30px 0 15px}* > body{background:url(//www.google.com/images/errors/robot.png) 100% 5px no-repeat;padding-right:205px}p{margin:11px 0 22px;overflow:hidden}ins{color:#777;text-decoration:none}a img{border:0}@media screen and (max-width:772px){body{background:none;margin-top:0;max-width:none;padding-right:0}}#logo{background:url(//www.google.com/images/branding/googlelogo/1x/googlelogo_color_150x54dp.png) no-repeat;margin-left:-5px}@media only screen and (min-resolution:192dpi){#logo{background:url(//www.google.com/images/branding/googlelogo/2x/googlelogo_color_150x54dp.png) no-repeat 0% 0%/100% 100%;-moz-border-image:url(//www.google.com/images/branding/googlelogo/2x/googlelogo_color_150x54dp.png) 0}}@media only screen and (-webkit-min-device-pixel-ratio:2){#logo{background:url(//www.google.com/images/branding/googlelogo/2x/googlelogo_color_150x54dp.png) no-repeat;-webkit-background-size:100% 100%}}#logo{display:inline-block;height:54px;width:150px}\n  </style>\n  <a href=//www.google.com/><span id=logo aria-label=Google></span></a>\n  <p><b>404.</b> <ins>That\xe2\x80\x99s an error.</ins>\n  <p>The requested URL <code>/one</code> was not found on this server.  <ins>That\xe2\x80\x99s all we know.</ins>\n'


class FuzzResultFactoryTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(FuzzResultFactoryTest, self).__init__(*args, **kwargs)
        self.maxDiff = 1000

    def test_baseline(self):
        options = CLParser(
            ["wfuzz", "-z", "range,1-1", "http://localhost:9000/FUZZ{first}"]
        ).parse_cl()
        options.compile_seeds()
        baseline = options["compiled_baseline"]

        self.assertEqual(baseline.description, "first")

        options = CLParser(
            [
                "wfuzz",
                "-z",
                "range,1-1",
                "-z",
                "range,2-2",
                "http://localhost:9000/FUZZ{first}/FUZ2Z{second}",
            ]
        ).parse_cl()
        options.compile_seeds()
        baseline = options["compiled_baseline"]

        self.assertEqual(baseline.description, "first - second")

    def test_from_conn(self):
        fr = FuzzRequest()
        fr.update_from_raw_http(
            raw_req, "https", raw_response_header, raw_response_body
        )

        self.assertEqual(fr.code, 404)
        self.assertEqual(fr.content.count("\n"), 11)


class FuzzRequestTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(FuzzRequestTest, self).__init__(*args, **kwargs)
        self.maxDiff = 1000

    def test_seturl(self):
        fr = FuzzRequest()

        fr.url = "http://www.wfuzz.org/"
        self.assertEqual(fr.url, "http://www.wfuzz.org/")
        self.assertEqual(fr.host, "www.wfuzz.org")
        self.assertEqual(fr.redirect_url, "http://www.wfuzz.org/")
        self.assertEqual(fr.scheme, "http")
        self.assertEqual(fr.path, "/")
        self.assertEqual(fr.follow, False)

        fr.url = "http://www.wfuzz.org"
        self.assertEqual(fr.url, "http://www.wfuzz.org/")

        fr.url = "www.wfuzz.org"
        self.assertEqual(fr.url, "http://www.wfuzz.org/")

        fr.url = "FUZZ://www.wfuzz.org/"
        self.assertEqual(fr.url, "FUZZ://www.wfuzz.org/")
        self.assertEqual(fr.scheme, "FUZZ")

        fr.url = "http://www.wfuzz.org/FUZZ"
        self.assertEqual(fr.url, "http://www.wfuzz.org/FUZZ")

        fr.url = "http://www.wfuzz.org/a"
        self.assertEqual(fr.url, "http://www.wfuzz.org/a")
        self.assertEqual(fr.path, "/a")

        fr.url = "http://www.wfuzz.org/a"
        self.assertEqual(sorted(str(fr).split("\n")), sorted(raw_req.split("\n")))

        fr.auth = {"method": "basic", "credentials": "admin:admin"}
        self.assertEqual(fr.auth, {"method": "basic", "credentials": "admin:admin"})

        fr.url = "FUZZ"
        self.assertEqual(fr.url, "FUZZ")
        self.assertEqual(fr.host, "")
        self.assertEqual(fr.redirect_url, "FUZZ")
        self.assertEqual(fr.scheme, "")
        self.assertEqual(fr.path, "FUZZ")
        self.assertEqual(fr.follow, False)

        fr.url = "http://www.wfuzz.org:80/a"
        self.assertEqual(fr.host, "www.wfuzz.org:80")

        fr.url = "https://www.wfuzz.org:80/a"
        self.assertEqual(fr.host, "www.wfuzz.org:80")

        fr.url = "www.wfuzz.org:80/a"
        self.assertEqual(fr.host, "www.wfuzz.org:80")

        fr.url = "www.wfuzz.org:80"
        self.assertEqual(fr.host, "www.wfuzz.org:80")

        fr.url = "www.wfuzz.org"
        self.assertEqual(fr.host, "www.wfuzz.org")

    def test_empy_post(self):
        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = ""
        self.assertEqual(fr.method, "POST")
        self.assertEqual(fr.params.post, {"": None})
        self.assertEqual(fr.params.raw_post, "")

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = {}
        self.assertEqual(fr.method, "POST")
        self.assertEqual(fr.params.post, {})
        self.assertEqual(fr.params.raw_post, "")

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = None
        self.assertEqual(fr.method, "GET")
        self.assertEqual(fr.params.post, {})
        self.assertEqual(fr.params.raw_post, None)

    def test_setpostdata(self):
        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = "a=1"
        self.assertEqual(fr.method, "POST")
        self.assertEqual(fr.params.raw_post, "a=1")
        self.assertEqual(fr.params.post, {"a": "1"})

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = "1"
        self.assertEqual(fr.method, "POST")
        self.assertEqual(fr.params.post, {"1": None})
        self.assertEqual(fr.params.raw_post, "1")

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = {"a": 1}
        self.assertEqual(fr.method, "POST")
        self.assertEqual(fr.params.post, {"a": "1"})
        self.assertEqual(fr.params.raw_post, "a=1")

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = {"a": "1"}
        self.assertEqual(fr.method, "POST")
        self.assertEqual(fr.params.post, {"a": "1"})
        self.assertEqual(fr.params.raw_post, "a=1")

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = "{'a': '1'}"
        self.assertEqual(fr.method, "POST")
        self.assertEqual(fr.params.post, {"{'a': '1'}": None})

    def test_setgetdata(self):
        fr = FuzzRequest()

        fr.url = "http://www.wfuzz.org/"
        fr.params.get = {"a": "1"}
        self.assertEqual(fr.method, "GET")
        self.assertEqual(fr.params.get, {"a": "1"})

    def test_allvars(self):
        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.get = {"a": "1", "b": "2"}
        fr.wf_allvars = "allvars"
        self.assertEqual(fr.wf_allvars_set, {"a": "1", "b": "2"})

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = {"a": "1", "b": "2"}
        fr.wf_allvars = "allpost"
        self.assertEqual(fr.wf_allvars_set, {"a": "1", "b": "2"})

        default_headers = dict(
            [
                ("Content-Type", "application/x-www-form-urlencoded"),
                ("User-Agent", "Wfuzz/{}".format(wfuzz_version)),
                ("Host", "www.wfuzz.org"),
            ]
        )

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.wf_allvars = "allheaders"
        self.assertEqual(fr.wf_allvars_set, default_headers)

    def test_cache_key(self):
        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        self.assertEqual(fr.to_cache_key(), "http://www.wfuzz.org/-")

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.get = {"a": "1", "b": "2"}
        self.assertEqual(fr.to_cache_key(), "http://www.wfuzz.org/-ga-gb")

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = {"c": "1", "d": "2"}
        self.assertEqual(fr.to_cache_key(), "http://www.wfuzz.org/-pc-pd")

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.get = {"a": "1", "b": "2"}
        fr.params.post = {"c": "1", "d": "2"}
        self.assertEqual(fr.to_cache_key(), "http://www.wfuzz.org/-ga-gb-pc-pd")

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.get = {"a": "1", "b": "2"}
        fr.params.post = {"a": "1", "b": "2"}
        self.assertEqual(fr.to_cache_key(), "http://www.wfuzz.org/-ga-gb-pa-pb")

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = "1"
        self.assertEqual(fr.to_cache_key(), "http://www.wfuzz.org/-p1")

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = ""
        self.assertEqual(fr.to_cache_key(), "http://www.wfuzz.org/-p")

    def test_cache_key_json_header_before(self):
        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = "1"
        fr.headers.request = {"Content-Type": "application/json"}

        self.assertEqual(fr.to_cache_key(), "http://www.wfuzz.org/-p1")

    def test_cache_key_json_header_after(self):
        fr = FuzzRequest()
        fr.headers.request = {"Content-Type": "application/json"}
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = "1"

        self.assertEqual(fr.to_cache_key(), "http://www.wfuzz.org/-p1")

    def test_cache_key_get_var(self):
        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/?a&b=1"

        self.assertEqual(fr.to_cache_key(), "http://www.wfuzz.org/-ga-gb")

    def test_get_vars(self):
        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/?a&b=1"
        self.assertEqual(fr.params.get, {"a": None, "b": "1"})

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/?"
        self.assertEqual(fr.params.get, {})

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        self.assertEqual(fr.params.get, {})

    def test_setpostdata_with_json(self):
        fr = FuzzRequest()
        fr.headers.request = {"Content-Type": "application/json"}
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = '{"string": "Foo bar","boolean": false}'
        self.assertEqual(fr.params.post, {"string": "Foo bar", "boolean": False})

        fr = FuzzRequest()
        fr.headers.request = {"Content-Type": "application/json"}
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = '{"array": [1,2]}'
        self.assertEqual(fr.params.post, {"array": [1, 2]})

    def test_post_bad_json(self):
        fr = FuzzRequest()
        fr.headers.request = {"Content-Type": "application/json"}
        fr.url = "http://www.wfuzz.org/"
        fr.params.post = "1"

        self.assertEqual(fr.method, "POST")
        self.assertEqual(fr.params.post, {"1": None})
        self.assertEqual(fr.params.raw_post, "1")

        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/"
        fr.headers.request = {"Content-Type": "application/json"}
        fr.params.post = "a=1"
        self.assertEqual(fr.method, "POST")
        self.assertEqual(fr.params.raw_post, "a=1")
        self.assertEqual(fr.params.post, {"a": "1"})


if __name__ == "__main__":
    unittest.main()



import unittest

# Python 2 and 3: urlib.parse

from wfuzz.fuzzobjects import FuzzResult
from wfuzz.fuzzrequest import FuzzRequest
from wfuzz.filters.ppfilter import FuzzResFilter


raw_req = """GET / HTTP/1.1
Host: www.wfuzz.org
User-Agent: curl/7.58.0
Accept: */*
"""

raw_resp = b"""HTTP/1.1 302 Found
Content-Type: text/html; charset=utf-8
Content-Language: en
Location: https://wfuzz.readthedocs.io/en/latest/
Vary: Accept-Language, Cookie
Server: nginx/1.14.0 (Ubuntu)
X-Fallback: True
X-Served: Django
X-Deity: web01
Date: Wed, 23 Jan 2019 21:43:59 GMT
Content-Length: 0
"""


class FilterTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(FilterTest, self).__init__(*args, **kwargs)
        self.maxDiff = 1000

    def get_filtered_fuzzrequest(self, filter_str):
        fr = FuzzRequest()
        fr.update_from_raw_http(raw_req, "http", raw_resp, b"")

        fuzz_res = FuzzResult(history=fr)

        ffilter = FuzzResFilter(filter_string=filter_str)
        ffilter.is_visible(fuzz_res)

        return fuzz_res

    def test_nonexisting(self):
        fr = FuzzRequest()
        fr.url = "http://www.wfuzz.org/path?param=1&param2=2"

        fuzz_res = FuzzResult(history=fr)

        with self.assertRaises(Exception) as context:
            ffilter = FuzzResFilter(filter_string="url=-'test'")
            ffilter.is_visible(fuzz_res)
            self.assertTrue("rsetattr: Can't set" in str(context.exception))

        with self.assertRaises(Exception) as context:
            ffilter = FuzzResFilter(filter_string="notthere=-'test'")
            ffilter.is_visible(fuzz_res)
            self.assertTrue("rgetattr: Can't get" in str(context.exception))

        with self.assertRaises(Exception) as context:
            ffilter = FuzzResFilter(filter_string="r.params.get.notthere=-'test'")
            ffilter.is_visible(fuzz_res)
            self.assertTrue("DotDict: Non-existing field" in str(context.exception))



import pytest
import wfuzz


@pytest.mark.parametrize(
    "session, expected_result",
    [
        (
            "-z range,0-4 http://127.0.0.1/FUZZ",
            {
                "allvars": None,
                "auth": {},
                "colour": False,
                "compiled_baseline": None,
                "compiled_dictio": None,
                "compiled_filter": None,
                "compiled_prefilter": [],
                "compiled_printer": None,
                "compiled_seed": None,
                "compiled_stats": None,
                "concurrent": 10,
                "conn_delay": 90,
                "connect_to_ip": None,
                "console_printer": "",
                "cookie": [],
                "delay": None,
                "dictio": None,
                "exec_mode": "api",
                "fields": [],
                "filter": "",
                "follow": False,
                "hc": [],
                "headers": [],
                "hh": [],
                "hl": [],
                "hs": None,
                "hw": [],
                "interactive": False,
                "iterator": None,
                "method": None,
                "no_cache": False,
                "payloads": [("range", {"default": "0-4", "encoder": None}, None)],
                "postdata": None,
                "prefilter": [],
                "previous": False,
                "printer": (None, None),
                "proxies": None,
                "recipe": [],
                "req_delay": 90,
                "retries": 3,
                "rlevel": 0,
                "dlevel": 4,
                "save": "",
                "sc": [],
                "scanmode": False,
                "script": "",
                "script_args": {},
                "seed_payload": False,
                "sh": [],
                "show_field": None,
                "sl": [],
                "ss": None,
                "sw": [],
                "transport": "http",
                "url": "http://127.0.0.1/FUZZ",
                "verbose": False,
            },
        )
    ],
)
def test_get_payload(session, expected_result):
    assert wfuzz.get_session(session).data == expected_result



import pytest

from wfuzz.helpers.obj_dic import DotDict
from wfuzz.helpers.obj_dyn import rgetattr


@pytest.fixture
def dotdict_ex1():
    return DotDict({"a": "1"})


@pytest.fixture
def dotdict_ex2():
    return DotDict({"a": "2"})


def test_operators(dotdict_ex1, dotdict_ex2):
    assert dotdict_ex1 == {"a": "1"}
    assert dotdict_ex1 + "test" == {"a": "1test"}
    assert "test" + dotdict_ex1 == {"a": "test1"}
    assert dotdict_ex1 + dotdict_ex2 == {"a": "2"}
    assert dotdict_ex2 + dotdict_ex1 == {"a": "1"}


def test_nonexisting_key_returns_none(dotdict_ex1):
    assert dotdict_ex1["anything"] == {}


def test_nonexisting_attr_returns_empty_dict(dotdict_ex1):
    assert rgetattr(dotdict_ex1, "anything") == {}



# slightly modified from
# https://gist.github.com/trungly/5889154

from http.server import HTTPServer
from http.server import SimpleHTTPRequestHandler
import urllib.parse


class GetHandler(SimpleHTTPRequestHandler):
    def do_HEAD(self):
        parsed_path = urllib.parse.urlparse(self.path)
        if parsed_path.path.startswith("/echo"):
            message = "\n".join(
                [
                    "CLIENT VALUES:",
                    "client_address=%s (%s)"
                    % (self.client_address, self.address_string()),
                    "command=%s" % self.command,
                    "path=%s" % self.path,
                    "real path=%s" % parsed_path.path,
                    "query=%s" % parsed_path.query,
                    "request_version=%s" % self.request_version,
                    "",
                    "HEADERS:",
                    "%s" % self.headers,
                ]
            )
            self.send_response(200)
            self.end_headers()
            self.wfile.write(message.encode("utf-8"))
        elif parsed_path.path.startswith("/redirect"):
            self.send_response(301)
            self.send_header("Location", "/echo")
            self.end_headers()
        else:
            SimpleHTTPRequestHandler.do_HEAD(self)

        return

    def do_GET(self):
        parsed_path = urllib.parse.urlparse(self.path)
        if parsed_path.path.startswith("/echo"):
            message = "\n".join(
                [
                    "CLIENT VALUES:",
                    "client_address=%s (%s)"
                    % (self.client_address, self.address_string()),
                    "command=%s" % self.command,
                    "path=%s" % self.path,
                    "real path=%s" % parsed_path.path,
                    "query=%s" % parsed_path.query,
                    "request_version=%s" % self.request_version,
                    "",
                    "HEADERS:",
                    "%s" % self.headers,
                ]
            )
            self.send_response(200)
            self.end_headers()
            self.wfile.write(message.encode("utf-8"))
        elif parsed_path.path.startswith("/redirect"):
            self.send_response(301)
            self.send_header("Location", "/echo")
            self.end_headers()
        else:
            SimpleHTTPRequestHandler.do_GET(self)

        return

    def do_POST(self):
        parsed_path = urllib.parse.urlparse(self.path)
        if parsed_path.path.startswith("/echo"):
            content_len = int(self.headers.get("content-length"))
            post_body = self.rfile.read(content_len).decode("utf-8")
            self.send_response(200)
            self.end_headers()

            message = "\n".join(
                [
                    "CLIENT VALUES:",
                    "client_address=%s (%s)"
                    % (self.client_address, self.address_string()),
                    "command=%s" % self.command,
                    "path=%s" % self.path,
                    "real path=%s" % parsed_path.path,
                    "query=%s" % parsed_path.query,
                    "request_version=%s" % self.request_version,
                    "",
                    "HEADERS:",
                    "%s" % self.headers,
                    "POST_DATA=%s" % post_body,
                    "",
                ]
            )

            self.wfile.write(message.encode("utf-8"))

        return


if __name__ == "__main__":
    server = HTTPServer(("0.0.0.0", 8000), GetHandler)
    server.serve_forever()



import pytest

# Python 2 and 3: urlib.parse

from wfuzz.fuzzrequest import FuzzRequest


@pytest.fixture
def full_fuzzreq(request):
    http_req, http_response = request.param
    fr = FuzzRequest()
    fr.update_from_raw_http(http_req, "http", http_response, None)

    return fr


@pytest.mark.parametrize(
    "full_fuzzreq, expected_result",
    [
        (
            (
                "GET /a HTTP/1.1\n"
                "Host: www.wfuzz.org\n"
                "Content-Type: application/x-www-form-urlencoded\n"
                "User-Agent: Wfuzz/2.1\n",
                "HTTP/1.0 301 Moved Permanently\n"
                "Server: SimpleHTTP/0.6 Python/3.6.5\n"
                "Date: Tue, 21 Apr 2020 21:10:53 GMT\n"
                "Location: /recursive_dir/a/\n",
            ),
            "http://www.wfuzz.org/recursive_dir/a/",
        ),
        (
            (
                "GET /a HTTP/1.1\n"
                "Host: www.wfuzz.org\n"
                "Content-Type: application/x-www-form-urlencoded\n"
                "User-Agent: Wfuzz/2.1\n",
                "HTTP/1.1 301 Moved Permanently\n"
                "Date: Fri, 24 Apr 2020 11:17:51 GMT\n"
                "Server: Apache/2.4.41 () OpenSSL/1.0.2k-fips\n"
                "Strict-Transport-Security: max-age=31536000; includeSubdomains; preload\n"
                "Location: https://www.wfuzz.org/\n"
                "Content-Type: text/html; charset=iso-8859-1\n",
            ),
            None,
        ),
        (
            (
                "GET /a HTTP/1.1\n"
                "Host: www.wfuzz.org\n"
                "Content-Type: application/x-www-form-urlencoded\n"
                "User-Agent: Wfuzz/2.1\n",
                "HTTP/1.0 404 File not found\n"
                "Server: SimpleHTTP/0.6 Python/3.6.5\n"
                "Date: Fri, 24 Apr 2020 12:37:54 GMT\n"
                "Connection: close\n"
                "Content-Type: text/html;charset=utf-8\n"
                "Content-Length: 469\n",
            ),
            None,
        ),
    ],
    indirect=["full_fuzzreq"],
)
def test_relative_url(full_fuzzreq, expected_result):
    assert full_fuzzreq.recursive_url == expected_result


@pytest.mark.parametrize(
    "full_fuzzreq, expected_result",
    [
        (
            (
                "GET /a HTTP/1.1\n"
                "Host: www.wfuzz.org\n"
                "Content-Type: application/x-www-form-urlencoded\n"
                "User-Agent: Wfuzz/2.1\n",
                "HTTP/1.0 301 Moved Permanently\n"
                "Server: SimpleHTTP/0.6 Python/3.6.5\n"
                "Date: Tue, 21 Apr 2020 21:10:53 GMT\n"
                "Location: /recursive_dir/a/\n",
            ),
            True,
        ),
        (
            (
                "GET /a HTTP/1.1\n"
                "Host: www.wfuzz.org\n"
                "Content-Type: application/x-www-form-urlencoded\n"
                "User-Agent: Wfuzz/2.1\n",
                "HTTP/1.1 301 Moved Permanently\n"
                "Date: Fri, 24 Apr 2020 11:17:51 GMT\n"
                "Server: Apache/2.4.41 () OpenSSL/1.0.2k-fips\n"
                "Strict-Transport-Security: max-age=31536000; includeSubdomains; preload\n"
                "Location: https://www.wfuzz.org/\n"
                "Content-Type: text/html; charset=iso-8859-1\n",
            ),
            False,
        ),
        (
            (
                "GET /a HTTP/1.1\n"
                "Host: www.wfuzz.org\n"
                "Content-Type: application/x-www-form-urlencoded\n"
                "User-Agent: Wfuzz/2.1\n",
                "HTTP/1.0 404 File not found\n"
                "Server: SimpleHTTP/0.6 Python/3.6.5\n"
                "Date: Fri, 24 Apr 2020 12:37:54 GMT\n"
                "Connection: close\n"
                "Content-Type: text/html;charset=utf-8\n"
                "Content-Length: 469\n",
            ),
            False,
        ),
        (
            (
                "GET /a/ HTTP/1.1\n"
                "Host: www.wfuzz.org\n"
                "Content-Type: application/x-www-form-urlencoded\n"
                "User-Agent: Wfuzz/2.1\n",
                "HTTP/1.0 200\n"
                "Server: SimpleHTTP/0.6 Python/3.6.5\n"
                "Date: Fri, 24 Apr 2020 12:37:54 GMT\n"
                "Connection: close\n"
                "Content-Type: text/html;charset=utf-8\n"
                "Content-Length: 469\n",
            ),
            True,
        ),
    ],
    indirect=["full_fuzzreq"],
)
def test_is_path(full_fuzzreq, expected_result):
    assert full_fuzzreq.is_path == expected_result



import unittest

from wfuzz.ui.console.clparser import CLParser


class CLParserTest(unittest.TestCase):
    def test_listplugins(self):
        with self.assertRaises(SystemExit) as cm:
            CLParser(["wfuzz", "-e", "iterators"]).parse_cl()

        self.assertEqual(cm.exception.code, 0)

    def test_ip_option(self):
        options = CLParser(["wfuzz", "--ip", "127.0.0.1"]).parse_cl()

        self.assertEqual(options.data["connect_to_ip"]["ip"], "127.0.0.1")
        self.assertEqual(options.data["connect_to_ip"]["port"], "80")

        options = CLParser(["wfuzz", "--ip", "127.0.0.1:22"]).parse_cl()

        self.assertEqual(options.data["connect_to_ip"]["ip"], "127.0.0.1")
        self.assertEqual(options.data["connect_to_ip"]["port"], "22")

        options = CLParser(["wfuzz", "--ip", "127.0.0.1:"]).parse_cl()

        self.assertEqual(options.data["connect_to_ip"]["ip"], "127.0.0.1")
        self.assertEqual(options.data["connect_to_ip"]["port"], "80")

        with self.assertRaises(Exception) as cm:
            options = CLParser(["wfuzz", "--ip", ":80"]).parse_cl()
        self.assertTrue("An IP must be specified" in str(cm.exception))

    def test_ze_zd_option(self):
        with self.assertRaises(Exception) as cm:
            options = CLParser(
                ["wfuzz", "-z", "range,0-10", "--zD", "0-10", "url"]
            ).parse_cl()
        self.assertTrue("exclusive" in str(cm.exception))

        options = CLParser(
            ["wfuzz", "-z", "range", "--zD", "0-1", "--zE", "md5", "url"]
        ).parse_cl()
        self.assertEqual(
            options.data["payloads"],
            [("range", {"default": "0-1", "encoder": ["md5"]}, None)],
        )

        options = CLParser(
            ["wfuzz", "-z", "range,0-1", "--zE", "md5", "url"]
        ).parse_cl()
        self.assertEqual(
            options.data["payloads"],
            [("range", {"default": "0-1", "encoder": ["md5"]}, None)],
        )

        options = CLParser(
            ["wfuzz", "-z", "range", "--zD", "0-1", "--zE", "md5", "url"]
        ).parse_cl()
        self.assertEqual(
            options.data["payloads"],
            [("range", {"default": "0-1", "encoder": ["md5"]}, None)],
        )

        options = CLParser(["wfuzz", "-z", "range", "--zD", "0-1"]).parse_cl()
        self.assertEqual(
            options.data["payloads"],
            [("range", {"default": "0-1", "encoder": None}, None)],
        )

        options = CLParser(["wfuzz", "-z", "range,0-1"]).parse_cl()
        self.assertEqual(
            options.data["payloads"],
            [("range", {"default": "0-1", "encoder": None}, None)],
        )



import pytest

from wfuzz.fuzzobjects import FuzzWord, FuzzWordType
from wfuzz.factories.fuzzfactory import SeedBuilderHelper
from wfuzz.ui.console.clparser import CLParser
from wfuzz.factories.fuzzresfactory import resfactory

from wfuzz.helpers.obj_dyn import rgetattr
import wfuzz.api


@pytest.mark.parametrize(
    "full_fuzzreq, expected_result",
    [
        (
            (
                "GET /FUZZ HTTP/1.1\n"
                "Host: www.wfuzz.org\n"
                "Content-Type: application/x-www-form-urlencoded\n"
                "User-Agent: Wfuzz/2.1\n",
                None,
            ),
            [
                {
                    "bl_value": None,
                    "field": None,
                    "full_bl": None,
                    "full_marker": "FUZZ",
                    "index": None,
                    "nonfuzz_marker": "",
                    "word": "FUZZ",
                }
            ],
        ),
        (
            (
                "GET /FUZZ{a_bl_value} HTTP/1.1\n"
                "Host: www.wfuzz.org\n"
                "Content-Type: application/x-www-form-urlencoded\n"
                "User-Agent: Wfuzz/2.1\n",
                None,
            ),
            [
                {
                    "bl_value": "a_bl_value",
                    "field": None,
                    "full_bl": "{a_bl_value}",
                    "full_marker": "FUZZ{a_bl_value}",
                    "index": None,
                    "nonfuzz_marker": "{a_bl_value}",
                    "word": "FUZZ",
                }
            ],
        ),
        (
            (
                "GET /FUZZ[url] HTTP/1.1\n"
                "Host: www.wfuzz.org\n"
                "Content-Type: application/x-www-form-urlencoded\n"
                "User-Agent: Wfuzz/2.1\n",
                None,
            ),
            [
                {
                    "bl_value": None,
                    "field": "url",
                    "full_bl": None,
                    "full_marker": "FUZZ[url]",
                    "index": None,
                    "nonfuzz_marker": "[url]",
                    "word": "FUZZ",
                }
            ],
        ),
        (
            (
                "GET /FUZZ/FUZ2Z[url] HTTP/1.1\n"
                "Host: www.wfuzz.org\n"
                "Content-Type: application/x-www-form-urlencoded\n"
                "User-Agent: Wfuzz/2.1\n",
                None,
            ),
            [
                {
                    "bl_value": None,
                    "field": None,
                    "full_bl": None,
                    "full_marker": "FUZZ",
                    "index": None,
                    "nonfuzz_marker": "",
                    "word": "FUZZ",
                },
                {
                    "bl_value": None,
                    "field": "url",
                    "full_bl": None,
                    "full_marker": "FUZ2Z[url]",
                    "index": "2",
                    "nonfuzz_marker": "[url]",
                    "word": "FUZ2Z",
                },
            ],
        ),
    ],
    indirect=["full_fuzzreq"],
)
def test_get_marker_dict(full_fuzzreq, expected_result):
    assert SeedBuilderHelper().get_marker_dict(full_fuzzreq) == expected_result


@pytest.mark.parametrize(
    "session_string, dictio, expected_field, expected_result",
    [
        (
            "wfuzz http://www.wfuzz.io/FUZZ",
            [FuzzWord("sub1", FuzzWordType.WORD)],
            "url",
            "http://www.wfuzz.io/sub1",
        ),
        (
            "wfuzz --basic FUZZ:FUZ2Z http://www.wfuzz.io/",
            (FuzzWord("sub1", FuzzWordType.WORD), FuzzWord("sub2", FuzzWordType.WORD)),
            "auth.credentials",
            "sub1:sub2",
        ),
        (
            "wfuzz --basic FUZZ:FUZ2Z http://www.wfuzz.io/",
            (FuzzWord("sub1", FuzzWordType.WORD), FuzzWord("sub2", FuzzWordType.WORD)),
            "auth.method",
            "basic",
        ),
    ],
)
def test_replace_markers(session_string, dictio, expected_field, expected_result):
    options = CLParser(session_string.split(" ")).parse_cl()
    options.compile_seeds()

    res = resfactory.create("fuzzres_from_options_and_dict", options, dictio)

    assert rgetattr(res.history, expected_field) == expected_result



import pytest


@pytest.mark.parametrize(
    "filter_string, expected_result",
    [
        ("h=28 or w=6 or l=2", True),
        ("r.params.get.param2='2'", True),
        ("r.headers.response.Location", "https://wfuzz.readthedocs.io/en/latest/"),
        ("r.headers.response.notthere", {}),
        ("r.params.get.notthere", {}),
        ("r.cookies.response.notthere", {}),
        ("r.cookies.response.notthere='something'", False),
        ("r.cookies.response.notthere~'something'", False),
        ("r.headers.request.Host", "www.wfuzz.org"),
        ("r.headers.request.host", "www.wfuzz.org"),
        ("r.headers.response.SeRVEr", "nginx/1.14.0 (Ubuntu)"),
        ("r.headers.response.server", "nginx/1.14.0 (Ubuntu)"),
        ("r.cookies.request.cookie1", "1"),
        ("r.cookies.request.cOOkiE1", "1"),
        ("r.cookies.response.name", "Nicholas"),
        ("r.cookies.response.nAMe", "Nicholas"),
        ("r.params.get.param1", "1"),
        ("r.params.get.pAraM1", "1"),
    ],
)
def test_filter_ret_values(
    filter_obj, example_full_fuzzres, filter_string, expected_result
):
    assert filter_obj.is_visible(example_full_fuzzres, filter_string) == expected_result


@pytest.mark.parametrize(
    "filter_string, expected_result",
    [
        ("r.headers.response.notthere", {}),
        ("r.params.get.notthere", {}),
        ("r.cookies.response.notthere", {}),
        ("r.cookies.response.notthere='something'", False),
    ],
)
def test_filter_ret_values_no_response(
    filter_obj, example_full_fuzzres_no_response, filter_string, expected_result
):
    assert (
        filter_obj.is_visible(example_full_fuzzres_no_response, filter_string)
        == expected_result
    )


@pytest.mark.parametrize(
    "filter_string, expected_result",
    [
        (
            "r.cookies.response.name|diff('test')",
            "--- prev\n\n+++ current\n\n@@ -1 +1 @@\n\n-test\n+Nicholas",
        ),
        ("r.cookies.response.nAMe|upper()", "NICHOLAS"),
        ("r.cookies.response.name|upper()", "NICHOLAS"),
        ("r.cookies.response.name|lower()", "nicholas"),
        ("r.cookies.response.name|startswith('N')", True),
        ("r.cookies.response.name|replace('N','n')", "nicholas"),
        ("'%2e%2e'|unquote()", ".."),
        ("'%2e%2f'|decode('urlencode')", "./"),
        ("'%%'|encode('urlencode')", "%25%25"),
    ],
)
def test_filter_operators(
    filter_obj, example_full_fuzzres, filter_string, expected_result
):
    assert filter_obj.is_visible(example_full_fuzzres, filter_string) == expected_result



import pytest

from wfuzz.helpers.obj_dic import CaseInsensitiveDict


@pytest.fixture
def case_dict():
    return CaseInsensitiveDict({"OnE": 1})


@pytest.mark.parametrize("key, expected_result", [("one", 1), ("oNe", 1)])
def test_key_get_item(case_dict, key, expected_result):
    assert case_dict[key] == expected_result
    assert case_dict.get(key) == expected_result


@pytest.mark.parametrize(
    "key, expected_result",
    [("One", True), ("OnE", True), ("one", True), ("onetwo", False)],
)
def test_key_in_item(case_dict, key, expected_result):
    assert (key in case_dict) == expected_result


def test_update():
    dd = CaseInsensitiveDict({})
    dd.update({"OnE": 1})

    assert dd["one"] == 1
    assert dd["oNe"] == 1


def test_key_in(case_dict):
    assert list(case_dict.keys()) == ["OnE"]



import os
import sys
import re
from setuptools import setup, find_packages

with open("README.md", "rb") as f:
    long_descr = f.read().decode("utf-8")


version = re.search(
    r'^__version__\s*=\s*"(.*)"',
    open('src/wfuzz/__init__.py').read(),
    re.M
).group(1)

docs_requires = [
    "Sphinx",
]

dev_requires = [
    'mock',
    'coverage',
    'codecov',
    'netaddr',  # tests/api/test_payload.py uses ipranges payload
    'pip-tools',
    'flake8==3.8.3',
    'black==19.10b0;python_version>"3.5"',
    'pytest',
]

install_requires = [
    'pycurl',
    'pyparsing<2.4.2;python_version<="3.4"',
    'pyparsing>=2.4*;python_version>="3.5"',
    'six',
    'configparser;python_version<"3.5"',
    'chardet',
]


if sys.platform.startswith("win"):
    install_requires += ["colorama>=0.4.0"]


try:
    os.symlink('../../docs/user/advanced.rst', 'src/wfuzz/advanced.rst')
    setup(
        name="wfuzz",
        packages=find_packages(where='src'),
        package_dir={'wfuzz': 'src/wfuzz'},
        include_package_data=True,
        package_data={'wfuzz': ['*.rst']},
        entry_points={
            'console_scripts': [
                'wfuzz = wfuzz.wfuzz:main',
                'wfpayload = wfuzz.wfuzz:main_filter',
                'wfencode = wfuzz.wfuzz:main_encoder',
            ],
            'gui_scripts': [
                'wxfuzz = wfuzz.wfuzz:main_gui',
            ]
        },
        version=version,
        description="Wfuzz - The web fuzzer",
        long_description=long_descr,
        long_description_content_type='text/markdown',
        author="Xavi Mendez (@x4vi_mendez)",
        author_email="xmendez@edge-security.com",
        url="http://wfuzz.org",
        license="GPLv2",
        install_requires=install_requires,
        extras_require={
            'dev': dev_requires,
            'docs': docs_requires,
        },
        python_requires=">=2.6",
        classifiers=(
            'Development Status :: 4 - Beta',
            'Natural Language :: English',
            'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
        ),
    )
finally:
    os.unlink('src/wfuzz/advanced.rst')



import pytest


@pytest.mark.parametrize(
    "fuzzres_from_url, filter_string, expected_result",
    [
        (
            "http://www.wfuzz.org/path?param=1&param2=2",
            "r.url=+'test'",
            "http://www.wfuzz.org/path?param=1&param2=2test",
        ),
        ("http://www.wfuzz.org/path?param=1&param2=2", "r.url:='test'", "http://test/"),
        (
            "http://www.wfuzz.org/path?param=1&param2=2",
            "r.url=-'test'",
            "testhttp://www.wfuzz.org/path?param=1&param2=2",
        ),
    ],
    indirect=["fuzzres_from_url"],
)
def test_url_set(filter_obj, fuzzres_from_url, filter_string, expected_result):
    filter_obj.is_visible(fuzzres_from_url, filter_string)
    assert fuzzres_from_url.history.url == expected_result


@pytest.mark.parametrize(
    "fuzzres_from_url, filter_string, expected_result",
    [("http://www.wfuzz.org/path?param", "r.params.all=+'test'", {"param": None})],
    indirect=["fuzzres_from_url"],
)
def test_params_set_no_value(
    filter_obj, fuzzres_from_url, filter_string, expected_result
):
    filter_obj.is_visible(fuzzres_from_url, filter_string)
    assert fuzzres_from_url.history.params.get == expected_result


@pytest.mark.parametrize(
    "fuzzres_from_url, filter_string, expected_result",
    [
        (
            "http://www.wfuzz.org/path?param=1&param2=2",
            "r.params.get.param=+'test'",
            {"param": "1test", "param2": "2"},
        ),
        (
            "http://www.wfuzz.org/path?param=1&param2=2",
            "r.params.get.param=-'test'",
            {"param": "test1", "param2": "2"},
        ),
        (
            "http://www.wfuzz.org/path?param=1&param2=2",
            "r.params.all=+'2'",
            {"param": "12", "param2": "22"},
        ),
        (
            "http://www.wfuzz.org/path?param=1&param2=2",
            "r.params.all:='2'",
            {"param": "2", "param2": "2"},
        ),
        (
            "http://www.wfuzz.org/path?param=1&param2=2",
            "r.params.get.notthere=-'2'",
            {"param": "1", "param2": "2"},
        ),
        (
            "http://www.wfuzz.org/path?param=1&param2=2",
            "r.params.get.notthere=+'2'",
            {"param": "1", "param2": "2"},
        ),
        (
            "http://www.wfuzz.org/path?param=1&param2=2",
            "r.params.get.notthere:='2'",
            {"notthere": "2", "param": "1", "param2": "2"},
        ),
    ],
    indirect=["fuzzres_from_url"],
)
def test_params_set(filter_obj, fuzzres_from_url, filter_string, expected_result):
    filter_obj.is_visible(fuzzres_from_url, filter_string)
    assert fuzzres_from_url.history.params.all == expected_result
